@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,6 +23,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
                         / ***/
 #include "qlifealgo.h"
 #include "hlifealgo.h"
+#include "jvnalgo.h"
+#include "ruletable_algo.h"
+#include "ruletreealgo.h"
+#include "generationsalgo.h"
 #include "readpattern.h"
 #include "util.h"
 #include "viewport.h"
@@ -31,6 +35,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <stdlib.h>
 #include <iostream>
 #include <cstdio>
+#include <string.h>
+#include <cstdlib>
 using namespace std ;
 viewport viewport(1000, 1000) ;
 /*
@@ -41,7 +47,8 @@ public:
    nullrender() {}
    virtual ~nullrender() {}
    virtual void killrect(int, int, int, int) {}
-   virtual void blit(int, int, int, int, int*, int=1) {}
+   virtual void pixblit(int, int, int, int, char*, int) {}
+   virtual void getcolors(unsigned char**, unsigned char**, unsigned char**) {}
 } ;
 nullrender renderer ;
 /*
@@ -58,8 +65,28 @@ public:
    virtual void beginprogress(const char *s) { abortprogress(0, s) ; }
    virtual bool abortprogress(double, const char *) ;
    virtual void endprogress() { abortprogress(1, "") ; }
+   virtual const char* getuserrules() { return "" ; }
+   virtual const char* getrulesdir() { return "Rules/" ; }
 } ;
 nullerrors nullerror ;
+/*
+ *   This is a "lifeerrors" that we can use to test some edge
+ *   conditions.  In this case the only real thing we use
+ *   it for is to check rendering during a progress dialog.
+ */
+class verbosestatus : public lifeerrors {
+public:
+   verbosestatus() {}
+   virtual void fatal(const char *) {}
+   virtual void warning(const char *) {}
+   virtual void status(const char *s) { cout << s << endl ; }
+   virtual void beginprogress(const char *) {}
+   virtual bool abortprogress(double, const char *) { return 0 ; }
+   virtual void endprogress() {}
+   virtual const char* getuserrules() { return "" ; }
+   virtual const char* getrulesdir() { return "Rules/" ; }
+} ;
+verbosestatus verbosestatus_instance ;
 char *filename ;
 lifealgo *imp = 0 ;
 struct options {
@@ -71,7 +98,10 @@ struct options {
 } ;
 bigint maxgen = -1, inc = 0 ;
 int maxmem = 256 ;
-int hyper, hashlife, render, autofit, quiet, popcount, progress ;
+int hyper, render, autofit, quiet, popcount, progress ;
+int hashlife ;
+char *algoName = (char *)"QuickLife" ;
+int verbose ;
 int stepthresh, stepfactor ;
 char *liferule = 0 ;
 char *outfilename = 0 ;
@@ -87,8 +117,10 @@ options options[] = {
   { "-q", "--quiet", "Don't show population; twice, don't show anything", 'b', &quiet },
   { "-r", "--rule", "Life rule to use", 's', &liferule },
   { "-h", "--hashlife", "Use Hashlife algorithm", 'b', &hashlife },
+  { "-a", "--algorithm", "Select algorithm by name", 's', &algoName },
   { "-o", "--output", "Output file (*.rle, *.mc, *.rle.gz, *.mc.gz)", 's',
                                                                &outfilename },
+  { "-v", "--verbose", "Verbose", 'b', &verbose },
   { "",   "--render", "Render (benchmarking)", 'b', &render },
   { "",   "--progress", "Render during progress dialog (debugging)", 'b', &progress },
   { "",   "--popcount", "Popcount (benchmarking)", 'b', &popcount },
@@ -116,7 +148,7 @@ void usage(const char *s) {
   fprintf(stderr, "Usage:  bgolly [options] patternfile\n") ;
   for (int i=0; options[i].shortopt; i++)
     fprintf(stderr, "%3s %-15s %s\n", options[i].shortopt, options[i].longopt,
-	    options[i].desc) ;
+            options[i].desc) ;
   if (s)
     lifefatal(s) ;
   exit(0) ;
@@ -142,7 +174,7 @@ void writepat(int fc) {
       FILE *f = fopen(thisfilename, "w") ;
       if (f == 0)
          lifefatal("Cannot open file for writing") ;
-      err = imp->writeNativeFormat(f, NULL) ;   // AKT: NULL means no comments
+      err = imp->writeNativeFormat(f, NULL) ;   // NULL means no comments
       fclose(f) ;
    } else {
       bigint t, l, b, r ;
@@ -179,18 +211,19 @@ struct cmdbase {
       int miny = iargs[1] ;
       int maxx = iargs[2] ;
       int maxy = iargs[3] ;
+      int v ;
       for (int y=miny; y<=maxy; y++) {
-	 for (int x=minx; x<=maxx; x++) {
-	    int dx = imp->nextcell(x, y) ;
-	    if (dx < 0)
-	       break ;
-	    if (x > 0 && (x + dx) < 0)
-	       break ;
-	    x += dx ;
-	    if (x > maxx)
-	       break ;
-	    nextloopinner(x, y) ;
-	 }
+         for (int x=minx; x<=maxx; x++) {
+            int dx = imp->nextcell(x, y, v) ;
+            if (dx < 0)
+               break ;
+            if (x > 0 && (x + dx) < 0)
+               break ;
+            x += dx ;
+            if (x > maxx)
+               break ;
+            nextloopinner(x, y) ;
+         }
       }
    }
    virtual void nextloopinner(int, int) {}
@@ -198,53 +231,53 @@ struct cmdbase {
       int iargn = 0 ;
       char sbuf[MAXCMDLENGTH+2] ;
       for (const char *rargs = args; *rargs; rargs++) {
-	 while (*cmdargs && *cmdargs <= ' ')
-	    cmdargs++ ;
-	 if (*cmdargs == 0) {
-	    lifewarning("Missing needed argument") ;
-	    return 0 ;
-	 }
-	 switch (*rargs) {
-	 case 'i':
-	   if (sscanf(cmdargs, "%d", iargs+iargn) != 1) {
-	     lifewarning("Missing needed integer argument") ;
-	     return 0 ;
-	   }
-	   iargn++ ;
-	   break ;
-	 case 'b':
-	   {
-	      int i = 0 ;
-	      for (i=0; cmdargs[i] > ' '; i++)
-		 sbuf[i] = cmdargs[i] ;
-	      sbuf[i] = 0 ;
-	      barg = bigint(sbuf) ;
-	   }
-	   break ;
-	 case 's':
-	   if (sscanf(cmdargs, "%s", sbuf) != 1) {
-	     lifewarning("Missing needed string argument") ;
-	     return 0 ;
-	   }
-	   sarg = strdup(sbuf) ;
-	   break ;
-	 default:
-	   lifefatal("Internal error in parseargs") ;
-	 }
-	 while (*cmdargs && *cmdargs > ' ')
-	   cmdargs++ ;
+         while (*cmdargs && *cmdargs <= ' ')
+            cmdargs++ ;
+         if (*cmdargs == 0) {
+            lifewarning("Missing needed argument") ;
+            return 0 ;
+         }
+         switch (*rargs) {
+         case 'i':
+           if (sscanf(cmdargs, "%d", iargs+iargn) != 1) {
+             lifewarning("Missing needed integer argument") ;
+             return 0 ;
+           }
+           iargn++ ;
+           break ;
+         case 'b':
+           {
+              int i = 0 ;
+              for (i=0; cmdargs[i] > ' '; i++)
+                 sbuf[i] = cmdargs[i] ;
+              sbuf[i] = 0 ;
+              barg = bigint(sbuf) ;
+           }
+           break ;
+         case 's':
+           if (sscanf(cmdargs, "%s", sbuf) != 1) {
+             lifewarning("Missing needed string argument") ;
+             return 0 ;
+           }
+           sarg = strdup(sbuf) ;
+           break ;
+         default:
+           lifefatal("Internal error in parseargs") ;
+         }
+         while (*cmdargs && *cmdargs > ' ')
+           cmdargs++ ;
       }
       return 1 ;
    }
    static void docmd(const char *cmdline) {
       for (cmdbase *cmd=list; cmd; cmd = cmd->next)
-	 if (strncmp(cmdline, cmd->verb, strlen(cmd->verb)) == 0 &&
-	     cmdline[strlen(cmd->verb)] <= ' ') {
-	    if (cmd->parseargs(cmdline+strlen(cmd->verb))) {
-	       cmd->doit() ;
-	    }
-	    return ;
-	 }
+         if (strncmp(cmdline, cmd->verb, strlen(cmd->verb)) == 0 &&
+             cmdline[strlen(cmd->verb)] <= ' ') {
+            if (cmd->parseargs(cmdline+strlen(cmd->verb))) {
+               cmd->doit() ;
+            }
+            return ;
+         }
       lifewarning("Didn't understand command") ;
    }
    cmdbase *next ;
@@ -299,7 +332,7 @@ struct helpcmd : public cmdbase {
    helpcmd() : cmdbase("help", "") {}
    virtual void doit() {
       for (cmdbase *cmd=list; cmd; cmd = cmd->next)
-	 cout << cmd->verb << " " << cmd->args << endl ;
+         cout << cmd->verb << " " << cmd->args << endl ;
    }
 } help_inst ;
 struct getcmd : public cmdbase {
@@ -312,8 +345,9 @@ struct getcmd : public cmdbase {
 struct getnextcmd : public cmdbase {
    getnextcmd() : cmdbase("getnext", "ii") {}
    virtual void doit() {
+     int v ;
      cout << "At " << iargs[0] << "," << iargs[1] << " next is " <<
-        imp->nextcell(iargs[0], iargs[1]) << endl ;
+        imp->nextcell(iargs[0], iargs[1], v) << endl ;
    }
 } getnext_inst ;
 vector<pair<int, int> > cutbuf ;
@@ -345,7 +379,7 @@ struct pastecmd : public cmdbase {
    pastecmd() : cmdbase("paste", "ii") {}
    virtual void doit() {
       for (unsigned int i=0; i<cutbuf.size(); i++)
-	 imp->setcell(cutbuf[i].first, cutbuf[i].second, 1) ;
+         imp->setcell(cutbuf[i].first, cutbuf[i].second, 1) ;
       cout << cutbuf.size() << " pixels pasted." << endl ;
    }
 } paste_inst ;
@@ -353,19 +387,31 @@ struct showcutcmd : public cmdbase {
    showcutcmd() : cmdbase("showcut", "") {}
    virtual void doit() {
       for (unsigned int i=0; i<cutbuf.size(); i++)
-	 cout << cutbuf[i].first << " " << cutbuf[i].second << endl ;
+         cout << cutbuf[i].first << " " << cutbuf[i].second << endl ;
    }
 } showcut_inst ;
+lifealgo *createUniverse() {
+   if (algoName == 0) {
+     if (hashlife)
+       algoName = (char *)"HashLife" ;
+     else
+       algoName = (char *)"QuickLife" ;
+   }
+   staticAlgoInfo *ai = staticAlgoInfo::byName(algoName) ;
+   if (ai == 0)
+      lifefatal("No such algorithm") ;
+   lifealgo *imp = (ai->creator)() ;
+   if (imp == 0)
+      lifefatal("Could not create universe") ;
+   imp->setMaxMemory(maxmem) ;
+   return imp ;
+}
 struct newcmd : public cmdbase {
    newcmd() : cmdbase("new", "") {}
    virtual void doit() {
      if (imp != 0)
-       delete imp ;
-     if (hashlife)
-       imp = new hlifealgo() ;
-     else
-       imp = new qlifealgo() ;
-     imp->setMaxMemory(maxmem) ;
+        delete imp ;
+     imp = createUniverse() ;
    }
 } new_inst ;
 struct sethashingcmd : public cmdbase {
@@ -380,6 +426,12 @@ struct setmaxmemcmd : public cmdbase {
       maxmem = iargs[0] ;
    }
 } setmaxmem_inst ;
+struct setalgocmd : public cmdbase {
+   setalgocmd() : cmdbase("setalgo", "s") {}
+   virtual void doit() {
+      algoName = sarg ;
+   }
+} setalgocmd_inst ;
 struct edgescmd : public cmdbase {
    edgescmd() : cmdbase("edges", "") {}
    virtual void doit() {
@@ -418,48 +470,54 @@ void runtestscript(const char *testscript) {
 }
 int main(int argc, char *argv[]) {
    cout << 
-    "This is bgolly " STRINGIFY(VERSION) " Copyright 2008 The Golly Gang."
+    "This is bgolly " STRINGIFY(VERSION) " Copyright 2009 The Golly Gang."
                                                             << endl << flush ;
+   qlifealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
+   hlifealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
+   jvnalgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
+   generationsalgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
+   ruletable_algo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
+   ruletreealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
    while (argc > 1 && argv[1][0] == '-') {
       argc-- ;
       argv++ ;
       char *opt = argv[0] ;
       int hit = 0 ;
       for (int i=0; options[i].shortopt; i++) {
-	if (strcmp(opt, options[i].shortopt) == 0 ||
-	    strcmp(opt, options[i].longopt) == 0) {
-	  switch (options[i].opttype) {
+        if (strcmp(opt, options[i].shortopt) == 0 ||
+            strcmp(opt, options[i].longopt) == 0) {
+          switch (options[i].opttype) {
 case 'i':
              if (argc < 2)
-	        lifefatal("Bad option argument") ;
+                lifefatal("Bad option argument") ;
              *(int *)options[i].data = atol(argv[1]) ;
-	     argc-- ;
-	     argv++ ;
+             argc-- ;
+             argv++ ;
              break ;
 case 'I':
              if (argc < 2)
-	        lifefatal("Bad option argument") ;
+                lifefatal("Bad option argument") ;
              *(bigint *)options[i].data = bigint(argv[1]) ;
-	     argc-- ;
-	     argv++ ;
+             argc-- ;
+             argv++ ;
              break ;
 case 'b':
              (*(int *)options[i].data) += 1 ;
              break ;
 case 's':
              if (argc < 2)
-	        lifefatal("Bad option argument") ;
+                lifefatal("Bad option argument") ;
              *(char **)options[i].data = argv[1] ;
-	     argc-- ;
-	     argv++ ;
+             argc-- ;
+             argv++ ;
              break ;
-	  }
-	  hit++ ;
-	  break ;
-	}
+          }
+          hit++ ;
+          break ;
+        }
       }
       if (!hit)
-	usage("Bad option given") ;
+        usage("Bad option given") ;
    }
    if (argc < 2 && !testscript)
      usage("No pattern argument given") ;
@@ -481,24 +539,23 @@ case 's':
       }
       if (outputgzip)
          lifefatal("Gzipped output files not supported yet") ;
-      if (outputismc && !hashlife)
-         lifefatal("Cannot write MC files if not using hash") ;
       if (strlen(outfilename) > 200)
          lifefatal("Output filename too long") ;
    }
-   if (hashlife)
-     imp = new hlifealgo() ;
-   else
-     imp = new qlifealgo() ;
+   imp = createUniverse() ;
    if (progress)
      lifeerrors::seterrorhandler(&nullerror) ;
+   else if (verbose) {
+     lifeerrors::seterrorhandler(&verbosestatus_instance) ;
+     hlifealgo::setVerbose(1) ;
+   }
    imp->setMaxMemory(maxmem) ;
    if (testscript) {
      if (argc > 1) {
        filename = argv[1] ;
        const char *err = readpattern(argv[1], *imp) ;
        if (err)
-	 lifefatal(err) ;
+         lifefatal(err) ;
      }
      runtestscript(testscript) ;
    }
@@ -515,16 +572,16 @@ case 's':
       if (quiet < 2) {
          cout << imp->getGeneration().tostring() ;
          if (!quiet)
-	   cout << ": " << imp->getPopulation().tostring() << endl ;
+           cout << ": " << imp->getPopulation().tostring() << endl ;
          else
-	   cout << endl ;
+           cout << endl ;
       }
       if (popcount)
          imp->getPopulation() ;
       if (autofit)
-	imp->fit(viewport, 1) ;
+        imp->fit(viewport, 1) ;
       if (render)
-	imp->draw(viewport, renderer) ;
+        imp->draw(viewport, renderer) ;
       if (maxgen >= 0 && imp->getGeneration() >= maxgen)
          break ;
       if (!hyper && maxgen > 0 && inc == 0) {
