@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -116,8 +116,8 @@ void hlifealgo::leafres(leaf *n) {
  *   We do now support garbage collection, but there are some routines we
  *   call frequently to help us.
  */
-#define node_hash(a,b,c,d) (((g_uintptr_t)d)+3*(((g_uintptr_t)c)+3*(((g_uintptr_t)b)+3*((g_uintptr_t)a)+3)))
-#define leaf_hash(a,b,c,d) ((d)+9*((c)+9*((b)+9*(a))))
+#define node_hash(a,b,c,d) (65537*(g_uintptr_t)(d)+257*(g_uintptr_t)(c)+17*(g_uintptr_t)(b)+5*(g_uintptr_t)(a))
+#define leaf_hash(a,b,c,d) (65537*(d)+257*(c)+17*(b)+5*(a))
 /*
  *   Resize the hash.
  */
@@ -300,7 +300,7 @@ node *hlifealgo::getres(node *n, int depth) {
        res = dorecurs(n->nw, n->ne, n->sw, n->se, depth) ;
      } else {
        res = (node *)dorecurs_leaf((leaf *)n->nw, (leaf *)n->ne,
-				   (leaf *)n->sw, (leaf *)n->se) ;
+                                   (leaf *)n->sw, (leaf *)n->se) ;
      }
    } else {
      if (halvesdone < 1000)
@@ -309,10 +309,10 @@ node *hlifealgo::getres(node *n, int depth) {
        res = dorecurs_half(n->nw, n->ne, n->sw, n->se, depth) ;
      } else if (ngens == 0) {
        res = (node *)dorecurs_leaf_quarter((leaf *)n->nw, (leaf *)n->ne,
-					   (leaf *)n->sw, (leaf *)n->se) ;
+                                           (leaf *)n->sw, (leaf *)n->se) ;
      } else {
        res = (node *)dorecurs_leaf_half((leaf *)n->nw, (leaf *)n->ne,
-					(leaf *)n->sw, (leaf *)n->se) ;
+                                        (leaf *)n->sw, (leaf *)n->se) ;
      }
    }
    pop(sp) ;
@@ -603,10 +603,10 @@ void hlifealgo::step() {
          lifefatal("bad increment") ;
       int downto = newpow2 ;
       if (ngens < newpow2)
-	 downto = ngens ;
+         downto = ngens ;
       if (newpow2 != ngens && cleareddownto > downto) {
          new_ngens(newpow2) ;
-	 cleareddownto = downto ;
+         cleareddownto = downto ;
       } else {
          ngens = newpow2 ;
       }
@@ -689,7 +689,7 @@ node *hlifealgo::zeronode(int depth) {
       zeronodea = (node **)realloc(zeronodea,
                                           nnzeros * sizeof(node *)) ;
       if (zeronodea == 0)
-	lifefatal("Out of memory (2).") ;
+        lifefatal("Out of memory (2).") ;
       alloced += (nnzeros - nzeros) * sizeof(node *) ;
       while (nzeros < nnzeros)
          zeronodea[nzeros++] = 0 ;
@@ -785,7 +785,7 @@ node *hlifealgo::setbit(node *n, int x, int y, int newstate, int depth) {
    } else {
       int w = 1 << depth ;
       if (depth > 31)
-	 w = 0 ;
+         w = 0 ;
       depth-- ;
       node **nptr ;
       if (x < 0) {
@@ -855,13 +855,13 @@ int hlifealgo::getbit(node *n, int x, int y, int depth) {
    } else {
       int w = 1 << depth ;
       if (depth > 31)
-	 w = 0 ;
+         w = 0 ;
       node *nptr ;
       depth-- ;
       if (x < 0) {
          if (y < 0)
             nptr = n->sw ;
-	 else
+         else
             nptr = n->nw ;
       } else {
          if (y < 0)
@@ -887,20 +887,20 @@ int hlifealgo::nextbit(node *n, int x, int y, int depth) {
       leaf *l = (leaf *)n ;
       int test = 0 ;
       if (y < 0)
-	test = (((l->sw >> (4 * (y & 3))) & 15) << 4) |
-	        ((l->se >> (4 * (y & 3))) & 15) ;
+        test = (((l->sw >> (4 * (y & 3))) & 15) << 4) |
+                ((l->se >> (4 * (y & 3))) & 15) ;
       else
-	test = (((l->nw >> (4 * (y & 3))) & 15) << 4) |
-	        ((l->ne >> (4 * (y & 3))) & 15) ;
+        test = (((l->nw >> (4 * (y & 3))) & 15) << 4) |
+                ((l->ne >> (4 * (y & 3))) & 15) ;
       test &= (1 << (4 - x)) - 1 ;
       if (test) {
-	int r = 0 ;
-	int b = 1 << (3 - x) ;
-	while ((test & b) == 0) {
-	  r++ ;
-	  b >>= 1 ;
-	}
-	return r ;
+        int r = 0 ;
+        int b = 1 << (3 - x) ;
+        while ((test & b) == 0) {
+          r++ ;
+          b >>= 1 ;
+        }
+        return r ;
       }
       return -1 ; // none found
    } else {
@@ -909,25 +909,25 @@ int hlifealgo::nextbit(node *n, int x, int y, int depth) {
       node *lft, *rght ;
       depth-- ;
       if (y < 0) {
-	lft = n->sw ;
-	rght = n->se ;
+        lft = n->sw ;
+        rght = n->se ;
       } else {
-	lft = n->nw ;
-	rght = n->ne ;
+        lft = n->nw ;
+        rght = n->ne ;
       }
       int r = 0 ;
       if (x < 0) {
-	int t = nextbit(lft, (x & (w-1)) - wh,
-			(y & (w - 1)) - wh, depth) ;
-	if (t >= 0)
-	  return t ;
-	r = -x ;
-	x = 0 ;
+        int t = nextbit(lft, (x & (w-1)) - wh,
+                        (y & (w - 1)) - wh, depth) ;
+        if (t >= 0)
+          return t ;
+        r = -x ;
+        x = 0 ;
       }
       int t = nextbit(rght, (x & (w-1)) - wh,
-		      (y & (w - 1)) - wh, depth) ;
+                      (y & (w - 1)) - wh, depth) ;
       if (t >= 0)
-	return r + t ;
+        return r + t ;
       return -1 ;
    }
 }
@@ -938,7 +938,9 @@ int hlifealgo::nextbit(node *n, int x, int y, int depth) {
  *   unhashed (but it's faster when unhashed).  We also turn on the inGC
  *   flag to inhibit popcount.
  */
-void hlifealgo::setcell(int x, int y, int newstate) {
+int hlifealgo::setcell(int x, int y, int newstate) {
+   if (newstate & ~1)
+      return -1 ;
    if (hashed) {
       clearstack() ;
       save(root) ;
@@ -969,6 +971,7 @@ void hlifealgo::setcell(int x, int y, int newstate) {
    if (hashed) {
       okaytogc = 0 ;
    }
+   return 0 ;
 }
 /*
  *   Our nonrecurse top-level bit getting routine.
@@ -994,7 +997,8 @@ int hlifealgo::getcell(int x, int y) {
  *   current universe, or -1 if none set to the right, or if
  *   the next set pixel is out of range.
  */
-int hlifealgo::nextcell(int x, int y) {
+int hlifealgo::nextcell(int x, int y, int &v) {
+   v = 1 ;
    y = - y ;
    int sx = x ;
    int sy = y ;
@@ -1019,11 +1023,11 @@ int hlifealgo::nextcell(int x, int y) {
       struct node tnode = *root ;
       int mdepth = depth ;
       while (mdepth > 30) {
-	 tnode.nw = tnode.nw->se ;
-	 tnode.ne = tnode.ne->sw ;
-	 tnode.sw = tnode.sw->ne ;
-	 tnode.se = tnode.se->nw ;
-	 mdepth-- ;
+         tnode.nw = tnode.nw->se ;
+         tnode.ne = tnode.ne->sw ;
+         tnode.sw = tnode.sw->ne ;
+         tnode.se = tnode.se->nw ;
+         mdepth-- ;
       }
       return nextbit(&tnode, x, y, mdepth) ;
    }
@@ -1191,7 +1195,7 @@ node *hlifealgo::save(node *n) {
       alloced += sizeof(node *)*(nstacksize-stacksize) ;
       stack = (node **)realloc(stack, nstacksize * sizeof(node *)) ;
       if (stack == 0)
-	lifefatal("Out of memory (3).") ;
+        lifefatal("Out of memory (3).") ;
       stacksize = nstacksize ;
    }
    stack[gsp++] = n ;
@@ -1224,11 +1228,12 @@ void hlifealgo::gc_mark(node *root, int invalidate) {
          gc_mark(root->ne, invalidate) ;
          gc_mark(root->sw, invalidate) ;
          gc_mark(root->se, invalidate) ;
-         if (root->res)
-	    if (invalidate)
-	      root->res = 0 ;
-	    else
-	      gc_mark(root->res, invalidate) ;
+         if (root->res) {
+            if (invalidate)
+              root->res = 0 ;
+            else
+              gc_mark(root->res, invalidate) ;
+         }
       }
    }
 }
@@ -1307,7 +1312,7 @@ void hlifealgo::clearcache(node *n, int depth, int clearto) {
       mark(n) ;
       if (depth > 3) {
          depth-- ;
-	 poller->poll() ;
+         poller->poll() ;
          clearcache(n->nw, depth, clearto) ;
          clearcache(n->ne, depth, clearto) ;
          clearcache(n->sw, depth, clearto) ;
@@ -1352,7 +1357,7 @@ void hlifealgo::new_ngens(int newval) {
    inGC = 1 ;
    for (i=0; i<hashprime; i++)
       for (p=hashtab[i]; p; p=clearmarkbit(p->next))
-	 if (is_node(p) && !marked(p))
+         if (is_node(p) && !marked(p))
             clearcache(p, node_depth(p), clearto) ;
    for (p=nodeblocks; p; p=p->next) {
       poller->poll() ;
@@ -1392,12 +1397,12 @@ const bigint &hlifealgo::getPopulation() {
    // since that will mess up the gc.
    if (!popValid) {
       if (inGC) {
-	needPop = 1 ;
-	return negone ;
+        needPop = 1 ;
+        return negone ;
       } else {
-	calcPopulation(root) ;
-	popValid = 1 ;
-	needPop = 0 ;
+        calcPopulation(root) ;
+        popValid = 1 ;
+        needPop = 0 ;
       }
    }
    return population ;
@@ -1453,8 +1458,8 @@ const char *hlifealgo::readmacrocell(char *line) {
       if (i >= indlen) {
          g_uintptr_t nlen = i + indlen + 10 ;
          ind = (node **)realloc(ind, sizeof(node*) * nlen) ;
-	 if (ind == 0)
-	   lifefatal("Out of memory (4).") ;
+         if (ind == 0)
+           lifefatal("Out of memory (4).") ;
          while (indlen < nlen)
             ind[indlen++] = 0 ;
       }
@@ -1485,18 +1490,26 @@ case '$':      x = 0 ;
 default:       return "Illegal character in readmacrocell." ;
             }
          }
-	 clearstack() ;
+         clearstack() ;
          ind[i++] = (node *)find_leaf(lnw, lne, lsw, lse) ;
       } else if (line[0] == '#') {
          switch (line[1]) {
          char *p, *pp ;
+         const char *err ;
          case 'R':
             p = line + 2 ;
             while (*p && *p <= ' ') p++ ;
             pp = p ;
             while (*pp > ' ') pp++ ;
             *pp = 0 ;
-            global_liferules.setrule(p) ;
+            
+            // AKT: need to check for B0-not-S8 rule
+            err = global_liferules.setrule(p);
+            if (err)
+               return err;
+            if (global_liferules.hasB0notS8)
+               return "B0-not-S8 rules are not allowed in HashLife.";
+            
             break ;
          case 'G':
             p = line + 2 ;
@@ -1511,16 +1524,16 @@ default:       return "Illegal character in readmacrocell." ;
          n = sscanf(line, "%d %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %d", &d, &nw, &ne, &sw, &se, &r) ;
          if (n < 0) // blank line; permit
             continue ;
-	 if (n == 0) {
-	    // conversion error in first argument; we allow only if the only
-	    // content on the line is whitespace.
-	    char *ws = line ;
-	    while (*ws && *ws <= ' ')
-	       ws++ ;
-	    if (*ws > 0)
-	       return "Parse error in macrocell format." ;
-	    continue ;
-	 }
+         if (n == 0) {
+            // conversion error in first argument; we allow only if the only
+            // content on the line is whitespace.
+            char *ws = line ;
+            while (*ws && *ws <= ' ')
+               ws++ ;
+            if (*ws > 0)
+               return "Parse error in macrocell format." ;
+            continue ;
+         }
          if (n < 5)
             // AKT: best not to use lifefatal here because user won't see any
             // error message when reading clipboard data starting with "[..."
@@ -1534,7 +1547,7 @@ default:       return "Illegal character in readmacrocell." ;
              se < 0 || se >= i || ind[se] == 0) {
             return "Node out of range in readmacrocell." ;
          }
-	 clearstack() ;
+         clearstack() ;
          root = ind[i++] = find_node(ind[nw], ind[ne], ind[sw], ind[se]) ;
          depth = d - 1 ;
       }
@@ -1553,11 +1566,19 @@ default:       return "Illegal character in readmacrocell." ;
 const char *hlifealgo::setrule(const char *s) {
    poller->bailIfCalculating() ;
    clearcache() ;
-   return global_liferules.setrule(s) ;
+   
+   // AKT: need to check for B0-not-S8 rule
+   const char* err = global_liferules.setrule(s);
+   if (err)
+      return err;
+   if (global_liferules.hasB0notS8)
+      return "B0-not-S8 rules are not allowed in HashLife.";
+   
+   return 0 ;
 }
 void hlifealgo::unpack8x8(unsigned short nw, unsigned short ne,
-			  unsigned short sw, unsigned short se,
-			  unsigned int *top, unsigned int *bot) {
+                          unsigned short sw, unsigned short se,
+                          unsigned int *top, unsigned int *bot) {
    *top = ((nw & 0xf000) << 16) | (((ne & 0xf000) | (nw & 0xf00)) << 12) |
           (((ne & 0xf00) | (nw & 0xf0)) << 8) |
           (((ne & 0xf0) | (nw & 0xf)) << 4) | (ne & 0xf) ;
@@ -1634,7 +1655,7 @@ g_uintptr_t hlifealgo::writecell_2p1(node *root, int depth) {
       thiscell = ++cellcounter ;
       // note:  we *must* not abort this prescan
       if ((cellcounter & 4095) == 0)
-	 lifeabortprogress(0, "Scanning tree") ;
+         lifeabortprogress(0, "Scanning tree") ;
       root->nw = (node *)thiscell ;
    } else {
       writecell_2p1(root->nw, depth-1) ;
@@ -1644,7 +1665,7 @@ g_uintptr_t hlifealgo::writecell_2p1(node *root, int depth) {
       thiscell = ++cellcounter ;
       // note:  we *must* not abort this prescan
       if ((cellcounter & 4095) == 0)
-	 lifeabortprogress(0, "Scanning tree") ;
+         lifeabortprogress(0, "Scanning tree") ;
       root->next = (node *)thiscell ;
    }
    return thiscell ;
@@ -1660,12 +1681,12 @@ g_uintptr_t hlifealgo::writecell_2p2(FILE *f, node *root, int depth) {
       return 0 ;
    if (depth == 2) {
       if (cellcounter + 1 != (g_uintptr_t)(root->nw))
-	 return (g_uintptr_t)(root->nw) ;
+         return (g_uintptr_t)(root->nw) ;
       thiscell = ++cellcounter ;
       if ((cellcounter & 4095) == 0) {
-	 unsigned long siz = ftell(f) ;
+         unsigned long siz = ftell(f) ;
          sprintf(progressmsg, "File size: %.2g MB", double((siz >> 20))) ;
-	 lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
+         lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
       }
       int i, j ;
       unsigned int top, bot ;
@@ -1686,21 +1707,21 @@ g_uintptr_t hlifealgo::writecell_2p2(FILE *f, node *root, int depth) {
       fputs("\n", f) ;
    } else {
       if (cellcounter + 1 > (g_uintptr_t)(root->next) || isaborted())
-	 return (g_uintptr_t)(root->next) ;
+         return (g_uintptr_t)(root->next) ;
       g_uintptr_t nw = writecell_2p2(f, root->nw, depth-1) ;
       g_uintptr_t ne = writecell_2p2(f, root->ne, depth-1) ;
       g_uintptr_t sw = writecell_2p2(f, root->sw, depth-1) ;
       g_uintptr_t se = writecell_2p2(f, root->se, depth-1) ;
       if (!isaborted() &&
-	  cellcounter + 1 != (g_uintptr_t)(root->next)) { // this should never happen
-	 lifefatal("Internal in writecell_2p2") ;
-	 return (g_uintptr_t)(root->next) ;
+          cellcounter + 1 != (g_uintptr_t)(root->next)) { // this should never happen
+         lifefatal("Internal in writecell_2p2") ;
+         return (g_uintptr_t)(root->next) ;
       }
       thiscell = ++cellcounter ;
       if ((cellcounter & 4095) == 0) {
-	 unsigned long siz = ftell(f) ;
+         unsigned long siz = ftell(f) ;
          sprintf(progressmsg, "File size: %.2g MB", double((siz >> 20))) ;
-	 lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
+         lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
       }
       root->next = (node *)thiscell ;
       fprintf(f, "%d %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %" PRIuPTR "\n", depth+1, nw, ne, sw, se) ;
@@ -1725,6 +1746,7 @@ const char *hlifealgo::writeNativeFormat(FILE *f, char *comments) {
       // write given comment line(s)
       fputs(comments, f) ;
    }
+   inGC = 1 ;
    /* this is the old way:
    cellcounter = 0 ;
    writecell(f, root, depth) ;
@@ -1737,7 +1759,22 @@ const char *hlifealgo::writeNativeFormat(FILE *f, char *comments) {
    writecell_2p2(f, root, depth) ;
    /* end new two-pass way */
    aftercalcpop2(root, depth, 0) ;
+   inGC = 0 ;
    return 0 ;
 }
-int hlifealgo::verbose = 0 ;
 char hlifealgo::statusline[120] ;
+static lifealgo *creator() { return new hlifealgo() ; }
+void hlifealgo::doInitializeAlgoInfo(staticAlgoInfo &ai) {
+   ai.setAlgorithmName("HashLife") ;
+   ai.setAlgorithmCreator(&creator) ;
+   ai.setDefaultBaseStep(8) ;
+   ai.setDefaultMaxMem(300) ;
+   ai.minstates = 2 ;
+   ai.maxstates = 2 ;
+   // init default color scheme
+   ai.defgradient = false;
+   ai.defr1 = ai.defg1 = ai.defb1 = 255;        // start color = white
+   ai.defr2 = ai.defg2 = ai.defb2 = 255;        // end color = white
+   ai.defr[0] = ai.defg[0] = ai.defb[0] = 48;   // 0 state = dark gray
+   ai.defr[1] = ai.defg[1] = ai.defb[1] = 255;  // 1 state = white
+}
