@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -28,7 +28,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "hlifealgo.h"
 #include <vector>
 #include <cstring>
-#include <iostream>
 #include <cstdio>
 #include <algorithm>
 using namespace std ;
@@ -37,52 +36,64 @@ const int logbmsize = 7 ;                 // 6=64x64  7=128x128  8=256x256
 const int bmsize = (1<<logbmsize) ;
 const int byteoff = (bmsize/8) ;
 const int ibufsize = (bmsize*bmsize/32) ;
-static unsigned int ibigbuf[ibufsize] ;   // a shared buffer for up to 256x256 pixels
+static unsigned int ibigbuf[ibufsize] ;   // a shared buffer for 128x128 pixels
 static unsigned char *bigbuf = (unsigned char *)ibigbuf ;
 
-// this version draws in XBM bitmap format
+// AKT: 128x128 pixmap where each pixel is 3 rgb bytes
+static unsigned char pixbuf[bmsize*bmsize*3];
+
+// AKT: rgb colors for cell states (see getcolors call)
+static unsigned char deadr, deadg, deadb;
+static unsigned char liver, liveg, liveb;
+
 static void drawpixel(int x, int y) {
-  bigbuf[(((bmsize-1)-y) << (logbmsize-3)) + (x >> 3)] |= (1 << (x & 7)) ;
+  bigbuf[(((bmsize-1)-y) << (logbmsize-3)) + (x >> 3)] |= (128 >> (x & 7)) ;
 }
+
 /*
  *   Draw a 4x4 area yielding 1x1, 2x2, or 4x4 pixels.
  */
 void draw4x4_1(unsigned short sw, unsigned short se,
                unsigned short nw, unsigned short ne, int llx, int lly) {
    unsigned char *p = bigbuf + ((bmsize-1+lly) << (logbmsize-3)) + ((-llx) >> 3) ;
-   int bit = 1 << ((-llx) & 0x7) ;
+   int bit = 128 >> ((-llx) & 0x7) ;
    if (sw) *p |= bit ;
-   if (se) *p |= (bit << 1) ;
+   if (se) *p |= (bit >> 1) ;
    p -= byteoff ;
    if (nw) *p |= bit ;
-   if (ne) *p |= (bit << 1) ;
+   if (ne) *p |= (bit >> 1) ;
 }
+
 void draw4x4_1(node *n, node *z, int llx, int lly) {
    unsigned char *p = bigbuf + ((bmsize-1+lly) << (logbmsize-3)) + ((-llx) >> 3) ;
-   int bit = 1 << ((-llx) & 0x7) ;
+   int bit = 128 >> ((-llx) & 0x7) ;
    if (n->sw != z) *p |= bit ;
-   if (n->se != z) *p |= (bit << 1) ;
+   if (n->se != z) *p |= (bit >> 1) ;
    p -= byteoff ;
    if (n->nw != z) *p |= bit ;
-   if (n->ne != z) *p |= (bit << 1) ;
+   if (n->ne != z) *p |= (bit >> 1) ;
 }
+
 static unsigned char compress4x4[256] ;
-static unsigned char rev8[256] ;
+static bool inited = false;
+
 void draw4x4_2(unsigned short bits1, unsigned short bits2, int llx, int lly) {
    unsigned char *p = bigbuf + ((bmsize-1+lly) << (logbmsize-3)) + ((-llx) >> 3) ;
-   int mask = (((-llx) & 0x4) ? 0xf0 : 0x0f) ;
+   int mask = (((-llx) & 0x4) ? 0x0f : 0xf0) ;
    int db = ((bits1 | (bits1 << 4)) & 0xf0f0) +
             ((bits2 | (bits2 >> 4)) & 0x0f0f) ;
    p[0] |= mask & compress4x4[db & 255] ;
    p[-byteoff] |= mask & compress4x4[db >> 8] ;
 }
+
 void draw4x4_4(unsigned short bits1, unsigned short bits2, int llx, int lly) {
    unsigned char *p = bigbuf + ((bmsize-1+lly) << (logbmsize-3)) + ((-llx) >> 3) ;
-   p[0] = rev8[((bits1 << 4) & 0xf0) + (bits2 & 0xf)] ;
-   p[-byteoff] = rev8[(bits1 & 0xf0) + ((bits2 >> 4) & 0xf)] ;
-   p[-2*byteoff] = rev8[((bits1 >> 4) & 0xf0) + ((bits2 >> 8) & 0xf)] ;
-   p[-3*byteoff] = rev8[((bits1 >> 8) & 0xf0) + ((bits2 >> 12) & 0xf)] ;
+   p[0] = ((bits1 << 4) & 0xf0) + (bits2 & 0xf) ;
+   p[-byteoff] = (bits1 & 0xf0) + ((bits2 >> 4) & 0xf) ;
+   p[-2*byteoff] = ((bits1 >> 4) & 0xf0) + ((bits2 >> 8) & 0xf) ;
+   p[-3*byteoff] = ((bits1 >> 8) & 0xf0) + ((bits2 >> 12) & 0xf) ;
 }
+
 void hlifealgo::clearrect(int minx, int miny, int w, int h) {
    // minx,miny is lower left corner
    if (w <= 0 || h <= 0)
@@ -110,9 +121,40 @@ void hlifealgo::renderbm(int x, int y) {
       rh *= pmag ;
    }
    ry = uviewh - ry - rh ;
-   renderer->blit(rx, ry, rw, rh, (int *)ibigbuf, pmag) ;
+   
+   // AKT: eventually we'll draw directly into pixbuf rather than bigbuf!!!
+   if (pmag > 1) {
+      // convert each bigbuf byte into 8 bytes of state data
+      int j = 0;
+      for (int i = 0; i < ibufsize * 4; i++) {
+         int byte = bigbuf[i];
+         for (int bit = 128; bit > 0; bit >>= 1) {
+            pixbuf[j++] = (byte & bit) ? 1 : 0;
+         }
+      }
+   } else {
+      // convert each bigbuf byte into 24 bytes of pixel data (8 * rgb)
+      int j = 0;
+      for (int i = 0; i < ibufsize * 4; i++) {
+         int byte = bigbuf[i];
+         for (int bit = 128; bit > 0; bit >>= 1) {
+            if (byte & bit) {
+               pixbuf[j++] = liver;
+               pixbuf[j++] = liveg;
+               pixbuf[j++] = liveb;
+            } else {
+               pixbuf[j++] = deadr;
+               pixbuf[j++] = deadg;
+               pixbuf[j++] = deadb;
+            }
+         }
+      }
+   }
+   renderer->pixblit(rx, ry, rw, rh, (char *)pixbuf, pmag);
+   
    memset(bigbuf, 0, sizeof(ibigbuf)) ;
 }
+
 /*
  *   Here, llx and lly are coordinates in screen pixels describing
  *   where the lower left pixel of the screen is.  Draw one node.
@@ -160,6 +202,7 @@ void hlifealgo::drawnode(node *n, int llx, int lly, int depth, node *z) {
       }
    }
 }
+
 /*
  *   Fill in the llxb and llyb bits from the viewport information.
  *   Allocate if necessary.  This arithmetic should be done carefully.
@@ -176,7 +219,7 @@ void hlifealgo::fill_ll(int d) {
    if (bitsreq2 > bitsreq)
       bitsreq = bitsreq2 ;
    if (bitsreq <= d)
-     bitsreq = d + 1 ; // need to access llxyb[d]
+      bitsreq = d + 1 ; // need to access llxyb[d]
    if (bitsreq > llsize) {
       if (llsize) {
          delete [] llxb ;
@@ -190,18 +233,16 @@ void hlifealgo::fill_ll(int d) {
    coor.first.tochararr(llxb, llbits) ;
    coor.second.tochararr(llyb, llbits) ;
 }
-static void init_rev8() {
+
+static void init_compress4x4() {
    int i;
-   for (i=0; i<8; i++) {
-      rev8[1<<i] = (unsigned char)(1<<(7-i)) ;
-      compress4x4[1<<i] = (unsigned char)(0x11 << ((7 - i) >> 1)) ;
-   }
+   for (i=0; i<8; i++)
+      compress4x4[1<<i] = (unsigned char)(0x11 << (i >> 1)) ;
    for (i=0; i<256; i++)
-      if (i & (i-1)) {
-         rev8[i] = rev8[i & (i-1)] + rev8[i & -i] ;
+      if (i & (i-1))
          compress4x4[i] = compress4x4[i & (i-1)] | compress4x4[i & -i] ;
-      }
 }
+
 /*
  *   This is the top-level draw routine that takes the root node.
  *   It maintains four nodes onto which the screen fits and uses the
@@ -212,11 +253,24 @@ static void init_rev8() {
  *   display an image.
  */
 void hlifealgo::draw(viewport &viewarg, liferender &rendererarg) {
-   if (rev8[255] == 0)
-      init_rev8() ;
+   if (!inited) {
+      init_compress4x4() ;
+      inited = true;
+   }
    memset(bigbuf, 0, sizeof(ibigbuf)) ;
    ensure_hashed() ;
    renderer = &rendererarg ;
+
+   // AKT: get cell colors
+   unsigned char *r, *g, *b;
+   renderer->getcolors(&r, &g, &b);
+   deadr = r[0];
+   deadg = g[0];
+   deadb = b[0];
+   liver = r[1];
+   liveg = g[1];
+   liveb = b[1];
+
    view = &viewarg ;
    uvieww = view->getwidth() ;
    uviewh = view->getheight() ;
@@ -247,35 +301,35 @@ void hlifealgo::draw(viewport &viewarg, liferender &rendererarg) {
       lly = (lly << 1) + llyb[i] ;
       if (llx > 2*maxd || lly > 2*maxd || llx < -2*maxd || lly < -2*maxd) {
          clearrect(0, 0, vieww, viewh) ;
-	 goto bail ;
+         goto bail ;
       }
    }
    /*  Find the lowest four we need to examine */
    while (d > 2 && d - mag >= 0 &&
-	  (d - mag > 28 || (1 << (d - mag)) > 2 * maxd)) {
+          (d - mag > 28 || (1 << (d - mag)) > 2 * maxd)) {
       llx = (llx << 1) + llxb[d] ;
       lly = (lly << 1) + llyb[d] ;
       if (llx >= 1) {
-	 if (lly >= 1) {
+         if (lly >= 1) {
             ne = ne->sw ;
             nw = nw->se ;
             se = se->nw ;
             sw = sw->ne ;
-	    lly-- ;
+            lly-- ;
          } else {
             ne = se->nw ;
             nw = sw->ne ;
             se = se->sw ;
             sw = sw->se ;
          }
-	 llx-- ;
+         llx-- ;
       } else {
-	 if (lly >= 1) {
+         if (lly >= 1) {
             ne = nw->se ;
             nw = nw->sw ;
             se = sw->ne ;
             sw = sw->nw ;
-	    lly-- ;
+            lly-- ;
          } else {
             ne = sw->ne ;
             nw = sw->nw ;
@@ -459,7 +513,7 @@ void hlifealgo::findedges(bigint *ptop, bigint *pleft, bigint *pbottom, bigint *
                   newv.push_back(t->se) ;
             }
          }
-	 sortunique(top, newv) ;
+         sortunique(top, newv) ;
          ymax += ymax ;
          if (!outer) {
             ymax.add_smallint(-2) ;
@@ -483,7 +537,7 @@ void hlifealgo::findedges(bigint *ptop, bigint *pleft, bigint *pbottom, bigint *
                   newv.push_back(t->ne) ;
             }
          }
-	 sortunique(bottom, newv) ;
+         sortunique(bottom, newv) ;
          ymin += ymin ;
          if (!outer) {
             ymin.add_smallint(2) ;
@@ -507,7 +561,7 @@ void hlifealgo::findedges(bigint *ptop, bigint *pleft, bigint *pbottom, bigint *
                   newv.push_back(t->sw) ;
             }
          }
-	 sortunique(right, newv) ;
+         sortunique(right, newv) ;
          xmax += xmax ;
          if (!outer) {
             xmax.add_smallint(-2) ;
@@ -531,7 +585,7 @@ void hlifealgo::findedges(bigint *ptop, bigint *pleft, bigint *pbottom, bigint *
                   newv.push_back(t->se) ;
             }
          }
-	 sortunique(left, newv) ;
+         sortunique(left, newv) ;
          xmin += xmin ;
          if (!outer) {
             xmin.add_smallint(2) ;
@@ -588,40 +642,40 @@ void hlifealgo::fit(viewport &view, int force) {
       currdepth-- ;
       if (currdepth == 1) { // we have leaf nodes; turn them into bitmasks
          topbm = getbitsfromleaves(top) & 0xff ;
-	 bottombm = getbitsfromleaves(bottom) & 0xff ;
-	 leftbm = getbitsfromleaves(left) >> 8 ;
-	 rightbm = getbitsfromleaves(right) >> 8 ;
+         bottombm = getbitsfromleaves(bottom) & 0xff ;
+         leftbm = getbitsfromleaves(left) >> 8 ;
+         rightbm = getbitsfromleaves(right) >> 8 ;
       }
       if (currdepth <= 1) {
-	 int sz = 1 << (currdepth + 2) ;
-	 int maskhi = (1 << sz) - (1 << (sz >> 1)) ;
-	 int masklo = (1 << (sz >> 1)) - 1 ;
+         int sz = 1 << (currdepth + 2) ;
+         int maskhi = (1 << sz) - (1 << (sz >> 1)) ;
+         int masklo = (1 << (sz >> 1)) - 1 ;
          ymax += ymax ;
-	 if ((topbm & maskhi) == 0) {
-	    ymax.add_smallint(-2) ;
-	    ysize-- ;
-	 } else {
-	    topbm >>= (sz >> 1) ;
-	 }
+         if ((topbm & maskhi) == 0) {
+            ymax.add_smallint(-2) ;
+            ysize-- ;
+         } else {
+            topbm >>= (sz >> 1) ;
+         }
          ymin += ymin ;
-	 if ((bottombm & masklo) == 0) {
-	   ymin.add_smallint(2) ;
-	   ysize-- ;
-	   bottombm >>= (sz >> 1) ;
-	 }
+         if ((bottombm & masklo) == 0) {
+           ymin.add_smallint(2) ;
+           ysize-- ;
+           bottombm >>= (sz >> 1) ;
+         }
          xmax += xmax ;
-	 if ((rightbm & masklo) == 0) {
-	    xmax.add_smallint(-2) ;
-	    xsize-- ;
-	    rightbm >>= (sz >> 1) ;
-	 }
+         if ((rightbm & masklo) == 0) {
+            xmax.add_smallint(-2) ;
+            xsize-- ;
+            rightbm >>= (sz >> 1) ;
+         }
          xmin += xmin ;
-	 if ((leftbm & maskhi) == 0) {
-	   xmin.add_smallint(2) ;
-	   xsize-- ;
-	 } else {
-	   leftbm >>= (sz >> 1) ;
-	 }
+         if ((leftbm & maskhi) == 0) {
+           xmin.add_smallint(2) ;
+           xsize-- ;
+         } else {
+           leftbm >>= (sz >> 1) ;
+         }
          xsize <<= 1 ;
          ysize <<= 1 ;
       } else {
