@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -194,7 +194,7 @@ void qlifealgo::uproot() {
      root = newsupertile(rootlev) ;
      if (rootlev > 1)
        root->flags = 0xf0000000 |
-	 (upchanging(oroot->flags) << (3 + (generation.odd()))) ;
+         (upchanging(oroot->flags) << (3 + (generation.odd()))) ;
      root->d[4] = oroot ;
      if (oroot != nullroot) {
        nullroots[rootlev] = nullroot = newsupertile(rootlev) ;
@@ -272,6 +272,11 @@ qlifealgo::~qlifealgo() {
  *   Set the max memory
  */
 void qlifealgo::setMaxMemory(int newmemlimit) {
+   // AKT: allow setting maxmemory to 0
+   if (newmemlimit == 0) {
+      maxmemory = 0 ;
+      return;
+   }
    if (newmemlimit < 10)
      newmemlimit = 10 ;
    else if (sizeof(maxmemory) <= 4 && newmemlimit > 4000)
@@ -540,7 +545,7 @@ int qlifealgo::p01(tile *p, tile *pr, tile *pd, tile *prd) {
  */
          p->c[i+2] |= (maskprev >> (6 - j)) & 0x1ff ;
          p->c[i+1] =
-	   (short)(((p->c[i+1] & 0x100) << 1) | (maskprev >> (21 - j))) ;
+           (short)(((p->c[i+1] & 0x100) << 1) | (maskprev >> (21 - j))) ;
       } else
          p->c[i+1] = 0 ;
 /*
@@ -630,7 +635,7 @@ int qlifealgo::p10(tile *plu, tile *pu, tile *pl, tile *p) {
             j++ ;
          }
          p->c[i+1] =
-	   (short)(((p->c[i+1] & 0x100) << 1) | (maskprev >> (14 + j))) ;
+           (short)(((p->c[i+1] & 0x100) << 1) | (maskprev >> (14 + j))) ;
          p->c[i] |= (maskprev >> j) & 0x1ff ;
       } else
          p->c[i+1] = 0 ;
@@ -658,7 +663,7 @@ void qlifealgo::markglobalchange(supertile *p, int lev) {
       }
    } else {
       if (p != nullroots[lev]) {
-	 p->flags |= 0xfffffff ;
+         p->flags |= 0xfffffff ;
          for (i=0; i<8; i++)
             markglobalchange(p->d[i], lev-1) ;
       }
@@ -674,7 +679,9 @@ void qlifealgo::markglobalchange() {
  *   We walk down the tree to the particular bit, setting changing flags as
  *   we go.
  */
-void qlifealgo::setcell(int x, int y, int newstate) {
+int qlifealgo::setcell(int x, int y, int newstate) {
+   if (newstate & ~1)
+      return -1 ;
    y = - y ;
    supertile *b ;
    tile *p ;
@@ -697,7 +704,7 @@ void qlifealgo::setcell(int x, int y, int newstate) {
    while (lev > 0) {
       int i, d = 1 ;
       if (lev & 1) {
-	 int s = (lev >> 1) + lev - 1 ;
+         int s = (lev >> 1) + lev - 1 ;
          i = (xdel >> s) & 7 ;
          s = (1 << (s + 5)) - 2 ;
          if ((xc & s) == ((odd) ? s : 0))
@@ -705,7 +712,7 @@ void qlifealgo::setcell(int x, int y, int newstate) {
          if ((yc & s) == ((odd) ? s : 0))
             d += d << 9 ;
       } else {
-	 int s = (lev >> 1) + lev - 3 ;
+         int s = (lev >> 1) + lev - 3 ;
          i = (ydel >> s) & 7 ;
          s = (1 << (s + 5)) - 2 ;
          if ((yc & s) == ((odd) ? s : 0))
@@ -755,6 +762,7 @@ void qlifealgo::setcell(int x, int y, int newstate) {
                                   &= ~(1 << (31 - (y & 7) * 4 - (x & 3))) ;
    }
    deltaforward = 0xffffffff ;
+   return 0 ;
 }
 /*
  *   This subroutine gets a bit at a particular location.
@@ -782,10 +790,10 @@ int qlifealgo::getcell(int x, int y) {
    while (lev > 0) {
       int i ;
       if (lev & 1) {
-	 int s = (lev >> 1) + lev - 1 ;
+         int s = (lev >> 1) + lev - 1 ;
          i = (xdel >> s) & 7 ;
       } else {
-	 int s = (lev >> 1) + lev - 3 ;
+         int s = (lev >> 1) + lev - 3 ;
          i = (ydel >> s) & 7 ;
       }
       if (b->d[i] == nullroots[lev-1])
@@ -815,7 +823,8 @@ int qlifealgo::getcell(int x, int y) {
 /**
  *   Similar but returns the distance to the next set cell horizontally.
  */
-int qlifealgo::nextcell(int x, int y) {
+int qlifealgo::nextcell(int x, int y, int &v) {
+   v = 1 ;
    y = - y ;
    int odd = generation.odd() ;
    if (odd) {
@@ -831,31 +840,31 @@ int qlifealgo::nextcell(int x, int y) {
 int qlifealgo::nextcell(int x, int y, supertile *n, int lev) {
    if (lev > 0) {
       if (n == nullroots[lev])
-	 return -1 ;
+         return -1 ;
       int xdel = (x >> 5) - minlow32 ;
       int ydel = (y >> 5) - minlow32 ;
       int i ;
       if (lev & 1) {
-	 int s = (lev >> 1) + lev - 1 ;
+         int s = (lev >> 1) + lev - 1 ;
          i = (xdel >> s) & 7 ;
-	 int r = 0 ;
-	 int off = (x & 31) + ((xdel & ((1 << s) - 1)) << 5) ;
-	 while (i < 8) {
-	    int t = nextcell(x, y, n->d[i], lev-1) ;
-	    if (t < 0) {
-	      r += (32 << s) - off ;
-	      x += (32 << s) - off ;
-	      off = 0 ;
-	    } else {
-	      return r + t ;
-	    }
-	    i++ ;
-	 }
-	 return -1 ;
+         int r = 0 ;
+         int off = (x & 31) + ((xdel & ((1 << s) - 1)) << 5) ;
+         while (i < 8) {
+            int t = nextcell(x, y, n->d[i], lev-1) ;
+            if (t < 0) {
+              r += (32 << s) - off ;
+              x += (32 << s) - off ;
+              off = 0 ;
+            } else {
+              return r + t ;
+            }
+            i++ ;
+         }
+         return -1 ;
       } else {
-	 int s = (lev >> 1) + lev - 3 ;
+         int s = (lev >> 1) + lev - 3 ;
          i = (ydel >> s) & 7 ;
-	 return nextcell(x, y, n->d[i], lev-1) ;
+         return nextcell(x, y, n->d[i], lev-1) ;
       }
    }
    x &= 31 ;
@@ -903,8 +912,8 @@ G_INT64 qlifealgo::find_set_bits(supertile *p, int lev, int gm1) {
                for (j=0; j<8; j++) {
                   k = pp->b[i]->d[j+gm1*8] ;
                   if (k)
-		    pop += bc[k & 255] + bc[(k >> 8) & 255] +
-		      bc[(k >> 16) & 255] + bc[(k >> 24) & 255] ;
+                    pop += bc[k & 255] + bc[(k >> 8) & 255] +
+                      bc[(k >> 16) & 255] + bc[(k >> 24) & 255] ;
                }
             }
          }
@@ -919,8 +928,8 @@ G_INT64 qlifealgo::find_set_bits(supertile *p, int lev, int gm1) {
             p->pop[gm1] = (long)pop ;
             p->flags &= ~(0x20000000 << gm1) ;
          } else {
-	    p->pop[gm1] = 0xfffffff ; // placeholder; *some* bits are set
-	 }
+            p->pop[gm1] = 0xfffffff ; // placeholder; *some* bits are set
+         }
       } else {
          pop = p->pop[gm1] ;
       }
@@ -1126,28 +1135,45 @@ const char *qlifealgo::setrule(const char *s) {
       // current rule doesn't have B0, or it has both B0 and S8
       ruletable = global_liferules.rule0 ;
       if (!global_liferules.vertically_symmetrical() &&
-	  !global_liferules.already_flipped()) {
-	// qlifealgo has an opposite interpretation of the orientation
-	// of ruletable than hlifealgo.  For vertically symmetrical
-	// rules, this doesn't matter.  This is a tad tricky because
-	// we want to turn both the input and the output of this function
-	// upside down.
-	for (int i=0; i<65536; i++) {
-	  int j = ((i & 0xf) << 12) +
-	    ((i & 0xf0) << 4) + ((i & 0xf00) >> 4) + ((i & 0xf000) >> 12) ;
-	  if (i <= j) {
-	    char fi = ruletable[i] ;
-	    char fj = ruletable[j] ;
-	    fi = ((fi & 0x30) >> 4) + ((fi & 0x3) << 4) ;
-	    fj = ((fj & 0x30) >> 4) + ((fj & 0x3) << 4) ;
-	    ruletable[i] = fj ;
-	    ruletable[j] = fi ;
-	  }
-	}
-	global_liferules.set_flipped() ;
+          !global_liferules.already_flipped()) {
+         // qlifealgo has an opposite interpretation of the orientation
+         // of ruletable than hlifealgo.  For vertically symmetrical
+         // rules, this doesn't matter.  This is a tad tricky because
+         // we want to turn both the input and the output of this function
+         // upside down.
+         for (int i=0; i<65536; i++) {
+            int j = ((i & 0xf) << 12) +
+                     ((i & 0xf0) << 4) + ((i & 0xf00) >> 4) + ((i & 0xf000) >> 12) ;
+            if (i <= j) {
+               char fi = ruletable[i] ;
+               char fj = ruletable[j] ;
+               fi = ((fi & 0x30) >> 4) + ((fi & 0x3) << 4) ;
+               fj = ((fj & 0x30) >> 4) + ((fj & 0x3) << 4) ;
+               ruletable[i] = fj ;
+               ruletable[j] = fi ;
+            }
+         }
+         global_liferules.set_flipped() ;
       }
    } else {
       // ruletable will be set in step() depending on gen parity
    }
    return p;
 }
+
+static lifealgo *creator() { return new qlifealgo() ; }
+
+void qlifealgo::doInitializeAlgoInfo(staticAlgoInfo &ai) {
+   ai.setAlgorithmName("QuickLife") ;
+   ai.setAlgorithmCreator(&creator) ;
+   ai.setDefaultBaseStep(10) ;
+   ai.setDefaultMaxMem(0) ;
+   ai.minstates = 2 ;
+   ai.maxstates = 2 ;
+   // init default color scheme
+   ai.defgradient = false;
+   ai.defr1 = ai.defg1 = ai.defb1 = 255;        // start color = white
+   ai.defr2 = ai.defg2 = ai.defb2 = 255;        // end color = white
+   ai.defr[0] = ai.defg[0] = ai.defb[0] = 48;   // 0 state = dark gray
+   ai.defr[1] = ai.defg[1] = ai.defb[1] = 255;  // 1 state = white
+}
