@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,11 +30,20 @@ const int logbmsize = 8 ;                 // *must* be 8 in this code
 const int bmsize = (1<<logbmsize) ;
 const int byteoff = (bmsize/8) ;
 const int ibufsize = (bmsize*bmsize/32) ;
-static unsigned int ibigbuf[ibufsize] ;   // shared buffer for up to 256x256 pixels
+static unsigned int ibigbuf[ibufsize] ;   // shared buffer for 256x256 pixels
 static unsigned char *bigbuf = (unsigned char *)ibigbuf ;
+
+// AKT: 256x256 pixmap where each pixel is 3 rgb bytes
+static unsigned char pixbuf[bmsize*bmsize*3];
+
+// AKT: rgb colors for cell states (see getcolors call)
+static unsigned char deadr, deadg, deadb;
+static unsigned char liver, liveg, liveb;
+
 void qlifealgo::renderbm(int x, int y) {
    renderbm(x, y, bmsize, bmsize) ;
 }
+
 void qlifealgo::renderbm(int x, int y, int xsize, int ysize) {
    // x,y is lower left corner
    int rx = x ;
@@ -48,21 +57,40 @@ void qlifealgo::renderbm(int x, int y, int xsize, int ysize) {
       rh *= pmag ;
    }
    ry = uviewh - ry - rh ;
-   renderer->blit(rx, ry, rw, rh, (int *)ibigbuf, pmag) ;
+
+   // AKT: eventually we'll draw directly into pixbuf rather than bigbuf!!!
+   if (pmag > 1) {
+      // convert each bigbuf byte into 8 bytes of state data
+      int j = 0;
+      for (int i = 0; i < ibufsize * 4; i++) {
+         int byte = bigbuf[i];
+         for (int bit = 128; bit > 0; bit >>= 1) {
+            pixbuf[j++] = (byte & bit) ? 1 : 0;
+         }
+      }
+   } else {
+      // convert each bigbuf byte into 24 bytes of pixel data (8 * rgb)
+      int j = 0;
+      for (int i = 0; i < ibufsize * 4; i++) {
+         int byte = bigbuf[i];
+         for (int bit = 128; bit > 0; bit >>= 1) {
+            if (byte & bit) {
+               pixbuf[j++] = liver;
+               pixbuf[j++] = liveg;
+               pixbuf[j++] = liveb;
+            } else {
+               pixbuf[j++] = deadr;
+               pixbuf[j++] = deadg;
+               pixbuf[j++] = deadb;
+            }
+         }
+      }
+   }
+   renderer->pixblit(rx, ry, rw, rh, (char *)pixbuf, pmag);
+
    memset(bigbuf, 0, sizeof(ibigbuf)) ;
 }
-static unsigned char rev8[256] ;
-static void init_rev8() {
-   int i;
-   for (i=0; i<8; i++) {
-      rev8[1<<i] = (unsigned char)(1<<(7-i)) ;
-   }
-   for (i=0; i<256; i++)
-      if (i & (i-1)) {
-         rev8[i] = rev8[i & (i-1)] + rev8[i & -i] ;
-      }
-}
-int minlevel;
+
 void qlifealgo::clearrect(int minx, int miny, int w, int h) {
    // minx,miny is lower left corner
    if (w <= 0 || h <= 0)
@@ -76,12 +104,14 @@ void qlifealgo::clearrect(int minx, int miny, int w, int h) {
    miny = uviewh - miny - h ;
    renderer->killrect(minx, miny, w, h) ;
 }
+
+static int minlevel;
 /*
  *   We cheat for now; we assume we can use 32-bit ints.  We can below
  *   a certain level; we'll deal with higher levels later.
  */
 void qlifealgo::BlitCells(supertile *p,
-			  int xoff, int yoff, int wd, int ht, int lev) {
+                          int xoff, int yoff, int wd, int ht, int lev) {
    int i, xinc=0, yinc=0, ypos, x, yy;
    int liveseen = 0 ;
    
@@ -142,14 +172,14 @@ void qlifealgo::BlitCells(supertile *p,
                            unsigned int v3 = (((v1 & 0x0f0f0f0f) << 4) |
                                                (v2 & 0x0f0f0f0f));
                            v2 = ((v1 & 0xf0f0f0f0) | ((v2 >> 4) & 0x0f0f0f0f));
-                           *p     = rev8[(unsigned char)v3];
-                           p[32]  = rev8[(unsigned char)v2];
-                           p[64]  = rev8[(unsigned char)(v3 >> 8)];
-                           p[96]  = rev8[(unsigned char)(v2 >> 8)];
-                           p[128] = rev8[(unsigned char)(v3 >> 16)];
-                           p[160] = rev8[(unsigned char)(v2 >> 16)];
-                           p[192] = rev8[(unsigned char)(v3 >> 24)];
-                           p[224] = rev8[(unsigned char)(v2 >> 24)];
+                           *p     = (unsigned char)v3;
+                           p[32]  = (unsigned char)v2;
+                           p[64]  = (unsigned char)(v3 >> 8);
+                           p[96]  = (unsigned char)(v2 >> 8);
+                           p[128] = (unsigned char)(v3 >> 16);
+                           p[160] = (unsigned char)(v2 >> 16);
+                           p[192] = (unsigned char)(v3 >> 24);
+                           p[224] = (unsigned char)(v2 >> 24);
                            
                            liveseen |= (1 << yy) ;
                         }
@@ -175,8 +205,7 @@ void qlifealgo::BlitCells(supertile *p,
    // to only render those portions that need to be rendered
    // (using this information).   -tom
    //
-   // draw the non-empty bitmap, scaling up if pmag > 1;
-   // note that bigbuf data must be in XBM format
+   // draw the non-empty bitmap, scaling up if pmag > 1
    renderbm(xoff, yoff) ;
 }
 
@@ -188,12 +217,12 @@ void qlifealgo::BlitCells(supertile *p,
 // the popcount child if needed.
 
 void qlifealgo::ShrinkCells(supertile *p,
-			    int xoff, int yoff, int wd, int ht, int lev) {
+                            int xoff, int yoff, int wd, int ht, int lev) {
    int i ;
    if (lev >= bmlev) {
       if (xoff >= vieww || xoff + wd < 0 || yoff >= viewh || yoff + ht < 0)
          // no part of this supertile/tile is visible
-	 return ;
+         return ;
       if (p == nullroots[lev]) {
          clearrect(xoff, yoff, wd, ht) ;
          return ;
@@ -204,12 +233,12 @@ void qlifealgo::ShrinkCells(supertile *p,
       }
    } else {
       if (p == nullroots[lev])
-	 return ;
+         return ;
    }
    int bminc = -1 << (logshbmsize-3) ;
    unsigned char *bm = bigbuf + (((shbmsize-1)-yoff+bmtop) << (logshbmsize-3)) +
                                   ((xoff-bmleft) >> 3) ;
-   int bit = 1 << ((xoff-bmleft) & 7) ;
+   int bit = 128 >> ((xoff-bmleft) & 7) ;
    // do recursion until we get to minimum level (which depends on mag)
    if (lev > minlevel) {
       int xinc = 0, yinc = 0 ;
@@ -245,7 +274,7 @@ void qlifealgo::ShrinkCells(supertile *p,
                int bbit = bit ;
                for (int j=0; j<8; j++) {
                   if (pp->d[j] != nullroots[lev-2]) {
-		     if (0 == (*bm & bbit)) {
+                     if (0 == (*bm & bbit)) {
                         supertile *ppp = pp->d[j] ;
                         if (lev > 2) {
                            if ( ppp->pop[oddgen] != 0 )
@@ -256,9 +285,9 @@ void qlifealgo::ShrinkCells(supertile *p,
                               *bm |= bbit ;
                         }
                      }
-		  }
+                  }
                   if ((j ^ (j + 1)) >> sh)
-                     bbit <<= 1 ;
+                     bbit >>= 1 ;
                }
             }
             if ((i ^ (i + 1)) >> sh)
@@ -303,7 +332,7 @@ void qlifealgo::ShrinkCells(supertile *p,
             if ( bt->d[kadd+4] | bt->d[kadd+5] | bt->d[kadd+6] | bt->d[kadd+7] |
                  bb->d[kadd+4] | bb->d[kadd+5] | bb->d[kadd+6] | bb->d[kadd+7] )
                // shrink 16x16 cells to 1 pixel
-               *bm |= bit << 1 ;
+               *bm |= bit >> 1 ;
          }
          bm += bminc ;
          if (t->b[2] != emptybrick || t->b[3] != emptybrick) {
@@ -318,7 +347,7 @@ void qlifealgo::ShrinkCells(supertile *p,
             if ( bt->d[kadd+4] | bt->d[kadd+5] | bt->d[kadd+6] | bt->d[kadd+7] |
                  bb->d[kadd+4] | bb->d[kadd+5] | bb->d[kadd+6] | bb->d[kadd+7] )
                // shrink 16x16 cells to 1 pixel
-               *bm |= bit << 1 ;
+               *bm |= bit >> 1 ;
          }
       }
       break;
@@ -333,11 +362,11 @@ void qlifealgo::ShrinkCells(supertile *p,
                brick *b = t->b[j];
                int k ;
                // examine the 8 slices (2 at a time) in the appropriate half-brick
+               int bbit = bit;
                for (k=0; k<8; k += 2) {
-                  if ( (b->d[k+kadd] | b->d[k+kadd+1]) ) *bm |= bit ;
-                  bit <<= 1 ;
+                  if ( (b->d[k+kadd] | b->d[k+kadd+1]) ) *bm |= bbit ;
+                  bbit >>= 1 ;
                }
-               bit >>= 4 ;
             }
             bm += bminc ;
          }
@@ -353,7 +382,7 @@ void qlifealgo::ShrinkCells(supertile *p,
             if (t->b[j] != emptybrick) {
                brick *b = t->b[j];
                int k ;
-               bit = 1 ;
+               bit = 128 ;
                // examine the 8 slices in the appropriate half-brick
                for (k=0; k<8; k++) {
                   unsigned int s = b->d[k+kadd];
@@ -362,7 +391,7 @@ void qlifealgo::ShrinkCells(supertile *p,
                      if (s & 0xFFFF0000) *bm |= bit ;
                      if (s & 0x0000FFFF) bm[bminc] |= bit ;
                   }
-                  bit <<= 1 ;
+                  bit >>= 1 ;
                }
             }
             bm += 2 * bminc ;
@@ -380,7 +409,7 @@ void qlifealgo::ShrinkCells(supertile *p,
             if (t->b[j] != emptybrick) {
                brick *b = t->b[j];
                int k ;
-               bit = 1 ;
+               bit = 128 ;
                // examine the 8 slices in the appropriate half-brick
                for (k=0; k<8; k++) {
                   unsigned int s = b->d[k+kadd];
@@ -390,23 +419,23 @@ void qlifealgo::ShrinkCells(supertile *p,
                      if (s & 0x00CC0000) bmm[bminc] |= bit ;
                      if (s & 0x0000CC00) bmm[bminc+bminc] |= bit ;
                      if (s & 0x000000CC) bmm[bminc+bminc+bminc] |= bit ;
-                     bit <<= 1 ;
+                     bit >>= 1 ;
                      if (s & 0x33000000) *bmm |= bit ;
                      if (s & 0x00330000) bmm[bminc] |= bit ;
                      if (s & 0x00003300) bmm[bminc+bminc] |= bit ;
                      if (s & 0x00000033) bmm[bminc+bminc+bminc] |= bit ;
-                     bit <<= 1 ;
+                     bit >>= 1 ;
                   } else {
-                     bit <<= 2 ;
+                     bit >>= 2 ;
                   }
-                  if (bit >= 256) {
+                  if (bit < 1) {
                      bmm++ ;
-                     bit = 1 ;
+                     bit = 128 ;
                   }
                }
-	       bmm -= 2 ;
+               bmm -= 2 ;
             }
-	    bmm += 4 * bminc ;
+            bmm += 4 * bminc ;
          }
       }
       break;
@@ -442,10 +471,19 @@ void qlifealgo::fill_ll(int d) {
    coor.second.tochararr(llyb, llbits) ;
 }
 void qlifealgo::draw(viewport &viewarg, liferender &renderarg) {
-   if (rev8[255] == 0)
-      init_rev8() ;
    memset(bigbuf, 0, sizeof(ibigbuf)) ;
    renderer = &renderarg ;
+
+   // AKT: get cell colors
+   unsigned char *r, *g, *b;
+   renderer->getcolors(&r, &g, &b);
+   deadr = r[0];
+   deadg = g[0];
+   deadb = b[0];
+   liver = r[1];
+   liveg = g[1];
+   liveb = b[1];
+
    view = &viewarg ;
    uvieww = view->getwidth() ;
    uviewh = view->getheight() ;
@@ -486,15 +524,15 @@ void qlifealgo::draw(viewport &viewarg, liferender &renderarg) {
       lly = (lly << 1) + llyb[i] ;
       if (llx > 2*maxd || lly > 2*maxd || llx < -2*maxd || lly < -2*maxd) {
          clearrect(0, 0, vieww, viewh) ;
-	 renderer = 0 ;
-	 view = 0 ;
-	 return ;
+         renderer = 0 ;
+         view = 0 ;
+         return ;
       }
    }
    /*  Find the lowest four we need to examine */
    int curlev = rootlev ;
    while (d > 8 && d - mag > 2 &&
-	  (d - mag > 28 || (1 << (d - mag)) > 32 * maxd)) {
+          (d - mag > 28 || (1 << (d - mag)) > 32 * maxd)) {
       // d is 5 + 3 * i for some positive i
       llx = (llx << 3) + (llxb[d-1] << 2) + (llxb[d-2] << 1) + llxb[d-3] ;
       lly = (lly << 3) + (llyb[d-1] << 2) + (llyb[d-2] << 1) + llyb[d-3] ;
@@ -510,36 +548,36 @@ void qlifealgo::draw(viewport &viewarg, liferender &renderarg) {
          yp = 7 ;
       if (xp == 7) {
          if (yp == 7) {
-	    ne = ne->d[0]->d[0] ;
-	    se = se->d[7]->d[0] ;
-	    nw = nw->d[0]->d[7] ;
-	    sw = sw->d[7]->d[7] ;
+            ne = ne->d[0]->d[0] ;
+            se = se->d[7]->d[0] ;
+            nw = nw->d[0]->d[7] ;
+            sw = sw->d[7]->d[7] ;
          } else {
-	    ne = se->d[yp+1]->d[0] ;
-	    se = se->d[yp]->d[0] ;
-	    nw = sw->d[yp+1]->d[7] ;
-	    sw = sw->d[yp]->d[7] ;
+            ne = se->d[yp+1]->d[0] ;
+            se = se->d[yp]->d[0] ;
+            nw = sw->d[yp+1]->d[7] ;
+            sw = sw->d[yp]->d[7] ;
          }
       } else {
          if (yp == 7) {
-	    ne = nw->d[0]->d[xp+1] ;
-	    se = sw->d[7]->d[xp+1] ;
-	    nw = nw->d[0]->d[xp] ;
-	    sw = sw->d[7]->d[xp] ;
+            ne = nw->d[0]->d[xp+1] ;
+            se = sw->d[7]->d[xp+1] ;
+            nw = nw->d[0]->d[xp] ;
+            sw = sw->d[7]->d[xp] ;
          } else {
-	    ne = sw->d[yp+1]->d[xp+1] ;
-	    se = sw->d[yp]->d[xp+1] ;
-	    nw = sw->d[yp+1]->d[xp] ;
-	    sw = sw->d[yp]->d[xp] ;
+            ne = sw->d[yp+1]->d[xp+1] ;
+            se = sw->d[yp]->d[xp+1] ;
+            nw = sw->d[yp+1]->d[xp] ;
+            sw = sw->d[yp]->d[xp] ;
          }
       }
       llx -= xp ;
       lly -= yp ;
       if (llx > 2*maxd || lly > 2*maxd || llx < -2*maxd || lly < -2*maxd) {
          clearrect(0, 0, vieww, viewh) ;
-	 renderer = 0 ;
-	 view = 0 ;
-	 return ;
+         renderer = 0 ;
+         view = 0 ;
+         return ;
       }
       d -= 3 ;
       curlev -= 2 ;
@@ -621,7 +659,7 @@ void qlifealgo::draw(viewport &viewarg, liferender &renderarg) {
  *   Find the subsupertiles with the smallest indices.
  */
 int qlifealgo::lowsub(vector<supertile*> &src, vector<supertile*> &dst,
-		      int lev) {
+                      int lev) {
   int lowlev = 7 ;
   dst.clear() ;
   supertile *z = nullroots[lev-1] ;
@@ -629,23 +667,23 @@ int qlifealgo::lowsub(vector<supertile*> &src, vector<supertile*> &dst,
     for (int i=0; i<(int)src.size(); i++) {
       supertile *p = src[i] ;
       for (int j=0; j<lowlev; j++)
-	if (p->d[j] != z && (p->d[j]->pop[oddgen])) {
-	  lowlev = j ;
-	  dst.clear() ;
-	}
+        if (p->d[j] != z && (p->d[j]->pop[oddgen])) {
+          lowlev = j ;
+          dst.clear() ;
+        }
       if (p->d[lowlev] != z && (p->d[lowlev]->pop[oddgen]))
-	dst.push_back(p->d[lowlev]) ;
+        dst.push_back(p->d[lowlev]) ;
     }
   } else {
     for (int i=0; i<(int)src.size(); i++) {
       supertile *p = src[i] ;
       for (int j=0; j<lowlev; j++)
-	if (p->d[j] != z && (((tile *)(p->d[j]))->flags & quickb)) {
-	  lowlev = j ;
-	  dst.clear() ;
-	}
+        if (p->d[j] != z && (((tile *)(p->d[j]))->flags & quickb)) {
+          lowlev = j ;
+          dst.clear() ;
+        }
       if (p->d[lowlev] != z && (((tile *)(p->d[lowlev]))->flags & quickb))
-	dst.push_back(p->d[lowlev]) ;
+        dst.push_back(p->d[lowlev]) ;
     }
   }
   return lowlev ;
@@ -654,7 +692,7 @@ int qlifealgo::lowsub(vector<supertile*> &src, vector<supertile*> &dst,
  *   Find the subsupertiles with the highest indices.
  */
 int qlifealgo::highsub(vector<supertile*> &src, vector<supertile*> &dst,
-		       int lev) {
+                       int lev) {
   int highlev = 0 ;
   dst.clear() ;
   supertile *z = nullroots[lev-1] ;
@@ -662,23 +700,23 @@ int qlifealgo::highsub(vector<supertile*> &src, vector<supertile*> &dst,
     for (int i=0; i<(int)src.size(); i++) {
       supertile *p = src[i] ;
       for (int j=7; j>highlev; j--)
-	if (p->d[j] != z && (p->d[j]->pop[oddgen])) {
-	  highlev = j ;
-	  dst.clear() ;
-	}
+        if (p->d[j] != z && (p->d[j]->pop[oddgen])) {
+          highlev = j ;
+          dst.clear() ;
+        }
       if (p->d[highlev] != z && (p->d[highlev]->pop[oddgen]))
-	dst.push_back(p->d[highlev]) ;
+        dst.push_back(p->d[highlev]) ;
     }
   } else {
     for (int i=0; i<(int)src.size(); i++) {
       supertile *p = src[i] ;
       for (int j=7; j>highlev; j--)
-	if (p->d[j] != z && (((tile *)(p->d[j]))->flags & quickb)) {
-	  highlev = j ;
-	  dst.clear() ;
-	}
+        if (p->d[j] != z && (((tile *)(p->d[j]))->flags & quickb)) {
+          highlev = j ;
+          dst.clear() ;
+        }
       if (p->d[highlev] != z && (((tile *)(p->d[highlev]))->flags & quickb))
-	dst.push_back(p->d[highlev]) ;
+        dst.push_back(p->d[highlev]) ;
     }
   }
   return highlev ;
@@ -687,22 +725,22 @@ int qlifealgo::highsub(vector<supertile*> &src, vector<supertile*> &dst,
  *   Find all nonzero sub-supertiles.
  */
 void qlifealgo::allsub(vector<supertile*> &src, vector<supertile*> &dst,
-		       int lev) {
+                       int lev) {
   dst.clear() ;
   supertile *z = nullroots[lev-1] ;
   if (lev > 1) {
     for (int i=0; i<(int)src.size(); i++) {
       supertile *p = src[i] ;
       for (int j=0; j<8; j++)
-	if (p->d[j] != z && (p->d[j]->pop[oddgen]))
-	  dst.push_back(p->d[j]) ;
+        if (p->d[j] != z && (p->d[j]->pop[oddgen]))
+          dst.push_back(p->d[j]) ;
     }
   } else {
     for (int i=0; i<(int)src.size(); i++) {
       supertile *p = src[i] ;
       for (int j=0; j<8; j++)
-	if (p->d[j] != z && (((tile *)(p->d[j]))->flags & quickb))
-	  dst.push_back(p->d[j]) ;
+        if (p->d[j] != z && (((tile *)(p->d[j]))->flags & quickb))
+          dst.push_back(p->d[j]) ;
     }
   }
 }
@@ -715,8 +753,8 @@ int qlifealgo::gethbitsfromleaves(vector<supertile *> v) {
     tile *p = (tile *)v[i] ;
     for (int j=0; j<4; j++)
       if (p->b[j] != emptybrick)
-	for (int k=0; k<8; k++)
-	  h[k] |= p->b[j]->d[k+kadd] ;
+        for (int k=0; k<8; k++)
+          h[k] |= p->b[j]->d[k+kadd] ;
   }
   int r = 0 ;
   for (i=0; i<8; i++) {
@@ -737,8 +775,8 @@ int qlifealgo::getvbitsfromleaves(vector<supertile *> vec) {
     tile *p = (tile *)vec[i] ;
     for (int j=0; j<4; j++)
       if (p->b[j] != emptybrick)
-	for (int k=0; k<8; k++)
-	  v[j] |= p->b[j]->d[k+kadd] ;
+        for (int k=0; k<8; k++)
+          v[j] |= p->b[j]->d[k+kadd] ;
   }
   int r = 0 ;
   for (i=3; i>=0; i--) {
@@ -746,7 +784,7 @@ int qlifealgo::getvbitsfromleaves(vector<supertile *> vec) {
     for (int j=0; j<8; j++) {
       r += r ;
       if (vv & (0xf << (4 * j)))
-	r++ ;
+        r++ ;
     }
   }
   return r ;
