@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -38,7 +38,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 bool getedges = false;              // find pattern edges?
 bigint top, left, bottom, right;    // the pattern edges
 
-#ifdef __WXMAC__
+#ifdef __APPLE__
 #define BUFFSIZE 4096      // 4K is best for Mac OS X
 #else
 #define BUFFSIZE 8192      // 8K is best for Windows and other platforms???
@@ -107,10 +107,12 @@ char *getline(char *line, int maxlinelen) {
    return line;
 }
 
+const char *SETCELLERROR = "Impossible; set cell error for state 1" ;
+
 // Read a text pattern like "...ooo$$$ooo" where '.', ',' and chars <= ' '
 // represent dead cells, '$' represents 10 dead cells, and all other chars
 // represent live cells.
-void readtextpattern(lifealgo &imp, char *line) {
+const char *readtextpattern(lifealgo &imp, char *line) {
    int x=0, y=0;
    char *p;
 
@@ -121,7 +123,9 @@ void readtextpattern(lifealgo &imp, char *line) {
          } else if (*p == '$') {
             x += 10;
          } else {
-            imp.setcell(x, y, 1);
+            if (imp.setcell(x, y, 1) < 0) {
+               return SETCELLERROR ;
+            }
             x++;
          }
       }
@@ -131,6 +135,7 @@ void readtextpattern(lifealgo &imp, char *line) {
    } while (getline(line, LINESIZE));
 
    if (getedges) bottom = y - 1;
+   return 0 ;
 }
 
 /*
@@ -143,13 +148,13 @@ void ParseXRLELine(char *line, int *xoff, int *yoff, bigint &gen) {
       while (*key && *key != ' ') key++;
       while (*key == ' ') key++;
       if (*key == 0) return;
-      
+
       // set value to pos of char after next '='
       char *value = key;
       while (*value && *value != '=') value++;
       if (*value == 0) return;
       value++;
-   
+
       if (strncmp(key, "Pos", 3) == 0) {
          // extract Pos=int,int
          sscanf(value, "%d,%d", xoff, yoff);
@@ -176,19 +181,20 @@ const char *readrle(lifealgo &imp, char *line) {
    int n=0, x=0, y=0 ;
    char *p ;
    char *ruleptr;
-   const char *errmsg = 0;
+   const char *errmsg;
    int wd=0, ht=0, xoff=0, yoff=0;
    bigint gen = bigint::zero;
    bool xrle = false;               // extended RLE format?
+   bool sawrule = false;            // saw explicit rule?
 
    // parse any #CXRLE line(s) at start
    while (strncmp(line, "#CXRLE", 6) == 0) {
       ParseXRLELine(line, &xoff, &yoff, gen);
       imp.setGeneration(gen);
       xrle = true;
-      if (getline(line, LINESIZE) == NULL) return errmsg;
+      if (getline(line, LINESIZE) == NULL) return 0;
    }
-   
+
    do {
       if (line[0] == '#') {
          if (line[1] == 'r') {
@@ -198,16 +204,22 @@ const char *readrle(lifealgo &imp, char *line) {
             p = ruleptr;
             while (*p > ' ') p++;
             *p = 0;
-            errmsg = imp.setrule(ruleptr) ;
+            errmsg = imp.setrule(ruleptr);
+            if (errmsg) return errmsg;
+            sawrule = true;
          }
-      } else if (line[0] == 'x') {
+         // there's a slight ambiguity here for extended RLE when a line
+         // starts with 'x'; we only treat it as a dimension line if the
+         // next char is whitespace or '=', since 'x' will only otherwise
+         // occur as a two-char token followed by an upper case alphabetic.
+      } else if (line[0] == 'x' && (line[1] <= ' ' || line[1] == '=')) {
          // extract wd and ht
          p = line;
          while (*p && *p != '=') p++; p++;
          sscanf(p, "%d", &wd);
          while (*p && *p != '=') p++; p++;
          sscanf(p, "%d", &ht);
-         
+
          /* we no longer center RLE pattern around 0,0
          if (!xrle) {
             xoff = -(wd / 2);
@@ -221,7 +233,7 @@ const char *readrle(lifealgo &imp, char *line) {
             bottom = yoff + ht - 1;
             right = xoff + wd - 1;
          }
-         
+
          while (*p && *p != 'r') p++;
          if (strncmp(p, "rule", 4) == 0) {
             p += 4;
@@ -231,34 +243,65 @@ const char *readrle(lifealgo &imp, char *line) {
             // remove any comma at end of rule
             if (p[-1] == ',') p--;
             *p = 0;
-            errmsg = imp.setrule(ruleptr) ;
+            errmsg = imp.setrule(ruleptr);
+            if (errmsg) return errmsg;
+            sawrule = true;
+         }
+
+         if (!sawrule) {
+            // if no rule given then try Conway's Life; if it fails then
+            // return error so Golly will look for matching algo
+            errmsg = imp.setrule("B3/S23");
+            if (errmsg) return errmsg;
          }
       } else {
          n = 0 ;
          for (p=line; *p; p++) {
-            if ('0' <= *p && *p <= '9') {
-               n = n * 10 + *p - '0' ;
+            char c = *p ;
+            if ('0' <= c && c <= '9') {
+               n = n * 10 + c - '0' ;
             } else {
                if (n == 0)
                   n = 1 ;
-               if (*p == 'b') {
+               if (c == 'b' || c == '.') {
                   x += n ;
-               } else if (*p == 'o') {
-                  while (n-- > 0)
-                     imp.setcell(xoff + x++, yoff + y, 1) ;
-               } else if (*p == '$') {
+               } else if (c == '$') {
                   x = 0 ;
                   y += n ;
-               } else if (*p == '!') {
-                  return errmsg;
+               } else if (c == '!') {
+                  return 0;
+               } else if (('o' <= c && c <= 'y') || ('A' <= c && c <= 'X')) {
+                  int state = -1 ;
+                  if (c == 'o')
+                     state = 1 ;
+                  else if (c < 'o') {
+                     state = c - 'A' + 1 ;
+                  } else {
+                     state = 24 * (c - 'p' + 1) ;
+                     p++ ;
+                     c = *p ;
+                     if ('A' <= c && c <= 'X') {
+                        state = state + c - 'A' + 1 ;
+                     } else {
+                        // return "Illegal multi-char state" ;
+                        // be more forgiving so we can read non-standard rle files like
+                        // those at http://home.interserv.com/~mniemiec/lifepage.htm
+                        state = 1 ;
+                        p-- ;
+                     }
+                  }
+                  while (n-- > 0) {
+                     if (imp.setcell(xoff + x++, yoff + y, state) < 0)
+                        return "Cell state out of range for this algorithm" ;
+                  }
                }
                n = 0 ;
             }
          }
       }
    } while (getline(line, LINESIZE));
-   
-   return errmsg;
+
+   return 0;
 }
 
 /*
@@ -270,15 +313,25 @@ const char *readpclife(lifealgo &imp, char *line) {
    int leftx = x ;
    char *p ;
    char *ruleptr;
-   const char *errmsg = 0;
+   const char *errmsg;
+   bool sawrule = false;            // saw explicit rule?
 
    do {
       if (line[0] == '#') {
          if (line[1] == 'P') {
+            if (!sawrule) {
+               // if no rule given then try Conway's Life; if it fails then
+               // return error so Golly will look for matching algo
+               errmsg = imp.setrule("B3/S23");
+               if (errmsg) return errmsg;
+               sawrule = true;      // in case there are many #P lines
+            }
             sscanf(line + 2, " %d %d", &x, &y) ;
             leftx = x ;
          } else if (line[1] == 'N') {
-            imp.setrule("B3/S23") ;
+            errmsg = imp.setrule("B3/S23");
+            if (errmsg) return errmsg;
+            sawrule = true;
          } else if (line[1] == 'R') {
             ruleptr = line;
             ruleptr += 2;
@@ -286,29 +339,34 @@ const char *readpclife(lifealgo &imp, char *line) {
             p = ruleptr;
             while (*p > ' ') p++;
             *p = 0;
-            errmsg = imp.setrule(ruleptr) ;
+            errmsg = imp.setrule(ruleptr);
+            if (errmsg) return errmsg;
+            sawrule = true;
          }
       } else if (line[0] == '-' || ('0' <= line[0] && line[0] <= '9')) {
          sscanf(line, "%d %d", &x, &y) ;
-         imp.setcell(x, y, 1) ;
+         if (imp.setcell(x, y, 1) < 0)
+            return SETCELLERROR ;
       } else if (line[0] == '.' || line[0] == '*') {
          for (p = line; *p; p++) {
-            if (*p == '*')
-               imp.setcell(x, y, 1) ;
+            if (*p == '*') {
+               if (imp.setcell(x, y, 1) < 0)
+                  return SETCELLERROR ;
+            }
             x++ ;
          }
          x = leftx ;
          y++ ;
       }
    } while (getline(line, LINESIZE));
-   
-   return errmsg;
+
+   return 0;
 }
 
 /*
  *   This routine reads David Bell's dblife format.
  */
-void readdblife(lifealgo &imp, char *line) {
+const char *readdblife(lifealgo &imp, char *line) {
    int n=0, x=0, y=0;
    char *p;
 
@@ -324,7 +382,9 @@ void readdblife(lifealgo &imp, char *line) {
                if (*p == '.') {
                   x += n;
                } else if (*p == 'O') {
-                  while (n-- > 0) imp.setcell(x++, y, 1);
+                  while (n-- > 0)
+                     if (imp.setcell(x++, y, 1) < 0)
+                        return SETCELLERROR ;
                } else {
                   // ignore dblife commands like "5k10h@"
                }
@@ -334,6 +394,106 @@ void readdblife(lifealgo &imp, char *line) {
          y++;
       }
    }
+   return 0;
+}
+
+//
+// Read Mirek Wojtowicz's MCell format.
+// See http://psoup.math.wisc.edu/mcell/ca_files_formats.html for details.
+//
+const char *readmcell(lifealgo &imp, char *line) {
+   int x=0, y=0;
+   char *p;
+   const char *errmsg;
+   bool sawrule = false;            // saw explicit rule?
+   bool extendedHL = false;         // special-case rule translation for extended HistoricalLife rules
+
+   while (getline(line, LINESIZE)) {
+      if (line[0] == '#') {
+         if (line[1] == 'L' && line[2] == ' ') {
+            if (!sawrule) {
+               // if no rule given then try Conway's Life; if it fails then
+               // return error so Golly will look for matching algo
+               errmsg = imp.setrule("B3/S23");
+               if (errmsg) return errmsg;
+               sawrule = true;
+            }
+
+            int n = 0;
+            for (p=line+3; *p; p++) {
+               char c = *p;
+               if ('0' <= c && c <= '9') {
+                  n = n * 10 + c - '0';
+               } else if (c > ' ') {
+                  if (n == 0)
+                     n = 1;
+                  if (c == '.') {
+                     x += n;
+                  } else if (c == '$') {
+                     x = 0;
+                     y += n;
+                  } else {
+                     int state = 0;
+                     if ('a' <= c && c <= 'j') {
+                        state = 24 * (c - 'a' + 1);
+                        p++;
+                        c = *p;
+                     }
+                     if ('A' <= c && c <= 'X') {
+                        state = state + c - 'A' + 1;
+                        if (extendedHL) {
+                           // adjust marked states for LifeHistory
+                           if (state == 8) state = 4;
+                           else if (state == 3) state = 5;
+                           else if (state == 5) state = 3;
+                        }
+                     } else {
+                        return "Illegal multi-char state";
+                     }
+                     while (n-- > 0) {
+                        if (imp.setcell(x++, y, state) < 0)
+                           return "Cell state out of range";
+                     }
+                  }
+                  n = 0;
+               }
+            }
+
+         // we allow lines like "#GOLLY WireWorld"
+         } else if (!sawrule && (strncmp(line, "#GOLLY", 6) == 0 ||
+                                 strncmp(line, "#RULE", 5) == 0) ) {
+             if (strncmp(line, "#RULE 1,0,1,0,0,0,1,0,0,0,0,0,0,2,2,1,1,2,2,2,2,2,0,2,2,2,1,2,2,2,2,2", 69) == 0) {
+                // standard HistoricalLife rule -- all states transfer directly to LifeHistory
+                if (strncmp(line, "#RULE 1,0,1,0,0,0,1,0,0,0,0,0,0,2,2,1,1,2,2,2,2,2,0,2,2,2,1,2,2,2,2,2,", 70) == 0) {
+                   // special case:  Brice Due's extended HistoricalLife rules have
+                   // non-contiguous states (State 8 but no State 4, 6, or 7)
+                   // that need translating to work in LifeHistory)
+                   extendedHL = true;
+                }
+                errmsg = imp.setrule("LifeHistory");
+                if (errmsg) return errmsg;
+                sawrule = true;
+             } else if (strncmp(line, "#RULE 1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,1", 40) == 0) {
+                errmsg = imp.setrule("B3/S23");
+                if (errmsg) errmsg = imp.setrule("Life");
+                if (errmsg) return errmsg;
+                sawrule = true;
+             } else {
+               char *ruleptr = line;
+               // skip "#GOLLY" or "#RULE"
+               ruleptr += line[1] == 'G' ? 6 : 5;
+               while (*ruleptr && *ruleptr <= ' ') ruleptr++;
+               p = ruleptr;
+               while (*p > ' ') p++;
+               *p = 0;
+               errmsg = imp.setrule(ruleptr);
+               if (errmsg) return errmsg;
+               sawrule = true;
+            }
+         }
+      }
+   }
+   return 0;
 }
 
 long getfilesize(const char *filename) {
@@ -363,7 +523,15 @@ const char *loadpattern(lifealgo &imp) {
       // ie. only change rule if an explicit rule is supplied
    } else {
       // reset rule to Conway's Life (default if explicit rule isn't supplied)
-      imp.setrule("B3/S23") ;
+      const char *err = imp.setrule("B3/S23") ;
+      if (err) {
+         // try "Life" in case given algo is RuleTable/RuleTree and a
+         // Life.table/tree file exists (nicer for loading lexicon patterns)
+         err = imp.setrule("Life");
+      }
+      // if given algo doesn't support B3/S23 or Life then the only sensible
+      // choice left is to use the algo's default rule
+      if (err) imp.setrule( imp.DefaultRule() ) ;
    }
 
    buffcount = 0;
@@ -395,13 +563,21 @@ const char *loadpattern(lifealgo &imp) {
          imp.findedges(&top, &left, &bottom, &right) ;
       }
 
+   } else if (line[0] == '#' && line[1] == 'M' && line[2] == 'C' &&
+              line[3] == 'e' && line[4] == 'l' && line[5] == 'l' ) {
+      errmsg = readmcell(imp, line) ;
+      imp.endofpattern() ;
+      if (getedges && !imp.isEmpty()) {
+         imp.findedges(&top, &left, &bottom, &right) ;
+      }
+
    } else if (line[0] == '#' || line[0] == 'x') {
       errmsg = readrle(imp, line) ;
       imp.endofpattern() ;
       // if getedges is true then readrle has set top,left,bottom,right
 
    } else if (line[0] == '!') {
-      readdblife(imp, line) ;
+      errmsg = readdblife(imp, line) ;
       imp.endofpattern() ;
       if (getedges && !imp.isEmpty()) {
          imp.findedges(&top, &left, &bottom, &right) ;
@@ -416,11 +592,11 @@ const char *loadpattern(lifealgo &imp) {
 
    } else {
       // read a text pattern like "...ooo$$$ooo"
-      readtextpattern(imp, line) ;
+      errmsg = readtextpattern(imp, line) ;
       imp.endofpattern() ;
       // if getedges is true then readtextpattern has set top,left,bottom,right
    }
-   
+
    lifeendprogress();
    return errmsg ;
 }
@@ -548,6 +724,22 @@ const char *readcomments(const char *filename, char **commptr)
          if (getline(line, LINESIZE) == NULL) break;
       }
 
+   } else if (line[0] == '#' && line[1] == 'M' && line[2] == 'C' &&
+              line[3] == 'e' && line[4] == 'l' && line[5] == 'l' ) {
+      // extract "#D ..." lines from MCell file
+      while (getline(line, LINESIZE)) {
+         if (line[0] != '#') break;
+         if (line[1] == 'L' && line[2] == ' ') break;
+         if (line[1] == 'D' && (line[2] == ' ' || line[2] == 0)) {
+            int linelen = strlen(line);
+            if (commlen + linelen + 1 > maxcommlen) break;
+            strncpy(cptr + commlen, line, linelen);
+            commlen += linelen;
+            cptr[commlen] = '\n';         // getline strips off eol char(s)
+            commlen++;
+         }
+      }
+
    } else if (line[0] == '#' || line[0] == 'x') {
       // extract comment lines from RLE file
       while (line[0] == '#') {
@@ -607,7 +799,7 @@ const char *readcomments(const char *filename, char **commptr)
    } else {
       // no comments in text pattern file???
    }
-   
+
    lifeendprogress();
    if (commlen == maxcommlen) commlen--;
    cptr[commlen] = 0;
