@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,18 +34,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #include "wxgolly.h"       // for wxGetApp, statusptr, viewptr, bigview
 #include "wxutils.h"       // for BeginProgress, GetString, etc
-#include "wxprefs.h"       // for maxhashmem, allowundo, etc
+#include "wxprefs.h"       // for allowundo, etc
 #include "wxrule.h"        // for ChangeRule
 #include "wxhelp.h"        // for LoadLexiconPattern
 #include "wxstatus.h"      // for statusptr->...
-#include "wxedit.h"        // for Selection
+#include "wxselect.h"      // for Selection
 #include "wxview.h"        // for viewptr->...
 #include "wxscript.h"      // for inscript, PassKeyToScript
 #include "wxmain.h"        // for MainFrame
 #include "wxundo.h"        // for undoredo->...
+#include "wxalgos.h"       // for *_ALGO, algo_type, CreateNewUniverse, etc
 #include "wxlayer.h"       // for currlayer, etc
+#include "wxrender.h"      // for DrawView
 
-// Control menu functions:
+// This module implements Control menu functions.
 
 // -----------------------------------------------------------------------------
 
@@ -62,8 +64,9 @@ bool MainFrame::SaveStartingPattern()
    currlayer->startdirty = currlayer->dirty;
    currlayer->startmag = viewptr->GetMag();
    viewptr->GetPos(currlayer->startx, currlayer->starty);
-   currlayer->startwarp = currlayer->warp;
-   currlayer->starthash = currlayer->hash;
+   currlayer->startbase = currlayer->currbase;
+   currlayer->startexpo = currlayer->currexpo;
+   currlayer->startalgo = currlayer->algtype;
    
    // if this layer is a clone then save some settings in other clones
    if (currlayer->cloneid > 0) {
@@ -74,7 +77,8 @@ bool MainFrame::SaveStartingPattern()
             cloneptr->startx = cloneptr->view->x;
             cloneptr->starty = cloneptr->view->y;
             cloneptr->startmag = cloneptr->view->getmag();
-            cloneptr->startwarp = cloneptr->warp;
+            cloneptr->startbase = cloneptr->currbase;
+            cloneptr->startexpo = cloneptr->currexpo;
          }
       }
    }
@@ -89,8 +93,9 @@ bool MainFrame::SaveStartingPattern()
    }
 
    // save starting pattern in tempstart file
-   if ( currlayer->hash ) {
-      // much faster to save hlife pattern in a macrocell file
+   //!!! use CanWriteFormat(MC_format)???
+   if ( currlayer->algo->hyperCapable() ) {
+      // much faster to save pattern in a macrocell file
       const char* err = WritePattern(currlayer->tempstart, MC_format, 0, 0, 0, 0);
       if (err) {
          statusptr->ErrorMessage(wxString(err,wxConvLocal));
@@ -98,7 +103,7 @@ bool MainFrame::SaveStartingPattern()
          return false;
       }
    } else {
-      // can only save qlife pattern if edges are within getcell/setcell limits
+      // can only save as RLE if edges are within getcell/setcell limits
       bigint top, left, bottom, right;
       currlayer->algo->findedges(&top, &left, &bottom, &right);      
       if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
@@ -163,18 +168,22 @@ void MainFrame::ResetPattern(bool resetundo)
       SavePendingChanges();
       currlayer->undoredo->RememberGenStart();
    }
+
+   // save current algo and rule
+   algo_type oldalgo = currlayer->algtype;
+   wxString oldrule = wxString(currlayer->algo->getrule(), wxConvLocal);
    
    // restore pattern and settings saved by SaveStartingPattern;
-   // first restore step size, hashing option and starting pattern
-   currlayer->warp = currlayer->startwarp;
-   currlayer->hash = currlayer->starthash;
+   // first restore algorithm
+   currlayer->algtype = currlayer->startalgo;
 
-   if ( !currlayer->startfile.IsEmpty() ) {
-      // restore pattern from startfile
-      LoadPattern(currlayer->startfile, wxEmptyString);
-   } else {
+   // restore starting pattern
+   if ( currlayer->startfile.IsEmpty() ) {
       // restore pattern from currfile
       LoadPattern(currlayer->currfile, wxEmptyString);
+   } else {
+      // restore pattern from startfile
+      LoadPattern(currlayer->startfile, wxEmptyString);
    }
    // gen count has been reset to startgen
    
@@ -185,7 +194,14 @@ void MainFrame::ResetPattern(bool resetundo)
    currlayer->currname = currlayer->startname;
    currlayer->algo->setrule(currlayer->startrule.mb_str(wxConvLocal));
    currlayer->dirty = currlayer->startdirty;
-   viewptr->SetPosMag(currlayer->startx, currlayer->starty, currlayer->startmag);
+   if (restoreview) {
+      viewptr->SetPosMag(currlayer->startx, currlayer->starty, currlayer->startmag);
+   }
+
+   // restore step size and set increment
+   currlayer->currbase = currlayer->startbase;
+   currlayer->currexpo = currlayer->startexpo;
+   SetGenIncrement();
 
    // if this layer is a clone then restore some settings in other clones
    if (currlayer->cloneid > 0) {
@@ -193,9 +209,12 @@ void MainFrame::ResetPattern(bool resetundo)
          Layer* cloneptr = GetLayer(i);
          if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
             cloneptr->currname = cloneptr->startname;
-            cloneptr->view->setpositionmag(cloneptr->startx, cloneptr->starty,
-                                           cloneptr->startmag);
-            cloneptr->warp = cloneptr->startwarp;
+            if (restoreview) {
+               cloneptr->view->setpositionmag(cloneptr->startx, cloneptr->starty,
+                                              cloneptr->startmag);
+            }
+            cloneptr->currbase = cloneptr->startbase;
+            cloneptr->currexpo = cloneptr->startexpo;
             // also synchronize dirty flags and update items in Layer menu
             cloneptr->dirty = currlayer->dirty;
             mainptr->UpdateLayerItem(i);
@@ -206,6 +225,12 @@ void MainFrame::ResetPattern(bool resetundo)
    // restore selection
    currlayer->currsel = currlayer->startsel;
 
+   // switch to default colors if algo/rule changed
+   wxString newrule = wxString(currlayer->algo->getrule(), wxConvLocal);
+   if (oldalgo != currlayer->algtype || oldrule != newrule) {
+      UpdateLayerColors();
+   }
+
    // update window title in case currname, rule or dirty flag changed;
    // note that UpdateLayerItem(currindex) gets called
    SetWindowTitle(currlayer->currname);
@@ -214,6 +239,7 @@ void MainFrame::ResetPattern(bool resetundo)
    if (allowundo && !currlayer->stayclean) {
       if (inscript) {
          // script called reset() so remember gen change
+         // (RememberGenStart was called above)
          currlayer->undoredo->RememberGenFinish();
       } else if (resetundo) {
          // wind back the undo history to the starting pattern
@@ -225,16 +251,14 @@ void MainFrame::ResetPattern(bool resetundo)
 // -----------------------------------------------------------------------------
 
 void MainFrame::RestorePattern(bigint& gen, const wxString& filename,
-                               bigint& x, bigint& y, int mag, int warp)
+                               bigint& x, bigint& y, int mag, int base, int expo)
 {
    // called to undo/redo a generating change
    if (gen == currlayer->startgen) {
       // restore starting pattern (false means don't call SyncUndoHistory)
       ResetPattern(false);
    } else {
-      // restore pattern in filename along with given pos/scale/speed settings
-      currlayer->warp = warp;
-
+      // restore pattern in given filename;
       // false means don't update status bar (algorithm should NOT change)
       LoadPattern(filename, wxEmptyString, false);
       
@@ -243,8 +267,15 @@ void MainFrame::RestorePattern(bigint& gen, const wxString& filename,
          // for some reason, so best to set correct gen count
          currlayer->algo->setGeneration(gen);
       }
+
+      // restore step size and set increment
+      currlayer->currbase = base;
+      currlayer->currexpo = expo;
+      SetGenIncrement();
+      
+      // restore position and scale, if allowed
+      if (restoreview) viewptr->SetPosMag(x, y, mag);
       
-      viewptr->SetPosMag(x, y, mag);
       UpdatePatternAndStatus();
    }
 }
@@ -281,7 +312,8 @@ const char* MainFrame::ChangeGenCount(const char* genstring, bool inundoredo)
       SavePendingChanges();
    }
 
-   if (!currlayer->hash && newgen.odd() != oldgen.odd()) {
+   //!!! need IsParityShifted() method???
+   if (currlayer->algtype == QLIFE_ALGO && newgen.odd() != oldgen.odd()) {
       // qlife stores pattern in different bits depending on gen parity,
       // so we need to create a new qlife universe, set its gen, copy the
       // current pattern to the new universe, then switch to that universe
@@ -290,8 +322,9 @@ const char* MainFrame::ChangeGenCount(const char* genstring, bool inundoredo)
       if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
          return "Pattern is too big to copy.";
       }
-      // create a new universe of same type
-      lifealgo* newalgo = CreateNewUniverse(currlayer->hash);
+      // create a new universe of same type and same rule
+      lifealgo* newalgo = CreateNewUniverse(currlayer->algtype);
+      newalgo->setrule(currlayer->algo->getrule());
       newalgo->setGeneration(newgen);
       // copy pattern
       if ( !viewptr->CopyRect(top.toint(), left.toint(), bottom.toint(), right.toint(),
@@ -362,11 +395,11 @@ void MainFrame::SetGeneration()
 
 void MainFrame::GoFaster()
 {
-   currlayer->warp++;
+   currlayer->currexpo++;
    SetGenIncrement();
    // only need to refresh status bar
    UpdateStatus();
-   if (generating && currlayer->warp < 0) {
+   if (generating && currlayer->currexpo < 0) {
       whentosee -= statusptr->GetCurrentDelay();
    }
 }
@@ -375,13 +408,13 @@ void MainFrame::GoFaster()
 
 void MainFrame::GoSlower()
 {
-   if (currlayer->warp > minwarp) {
-      currlayer->warp--;
+   if (currlayer->currexpo > minexpo) {
+      currlayer->currexpo--;
       SetGenIncrement();
       // only need to refresh status bar
       UpdateStatus();
-      if (generating && currlayer->warp < 0) {
-         if (currlayer->warp == -1) {
+      if (generating && currlayer->currexpo < 0) {
+         if (currlayer->currexpo == -1) {
             // need to initialize whentosee rather than increment it
             whentosee = stopwatch->Time() + statusptr->GetCurrentDelay();
          } else {
@@ -395,23 +428,53 @@ void MainFrame::GoSlower()
 
 // -----------------------------------------------------------------------------
 
+void MainFrame::SetBaseStep()
+{
+   int i;
+   if ( GetInteger(_("Set Base Step"),
+                   _("Temporarily change the current base step:"),
+                   currlayer->currbase, 2, MAX_BASESTEP, &i) ) {
+      currlayer->currbase = i;
+      SetGenIncrement();
+      UpdateStatus();
+   }
+}
+
+// -----------------------------------------------------------------------------
+
 void MainFrame::DisplayPattern()
 {
-   // this routine is only used by GeneratePattern();
-   // it's similar to UpdatePatternAndStatus() but if tiled windows exist
-   // it only updates the current tile if possible; ie. it's not a clone
+   // this routine is similar to UpdatePatternAndStatus() but if tiled windows
+   // exist it only updates the current tile if possible; ie. it's not a clone
    // and tile views aren't synchronized
    
    if (!IsIconized()) {
       if (tilelayers && numlayers > 1 && !syncviews && currlayer->cloneid == 0) {
          // only update the current tile
-         viewptr->Refresh(false);
-         viewptr->Update();
+         #if defined(__WXMAC__) && (MAC_OS_X_VERSION_MIN_REQUIRED != 1030)
+            // Refresh + Update is too slow on Mac
+            wxClientDC dc(viewptr);
+            DrawView(dc, viewptr->tileindex);
+         #else
+            viewptr->Refresh(false);
+            viewptr->Update();
+         #endif
       } else {
          // update main viewport window, possibly including all tile windows
          // (tile windows are children of bigview)
-         bigview->Refresh(false);
-         bigview->Update();
+         if (numlayers > 1 && (stacklayers || tilelayers)) {
+            bigview->Refresh(false);
+            bigview->Update();
+         } else {
+            #if defined(__WXMAC__) && (MAC_OS_X_VERSION_MIN_REQUIRED != 1030)
+               // Refresh + Update is too slow on Mac
+               wxClientDC dc(viewptr);
+               DrawView(dc, viewptr->tileindex);
+            #else
+               viewptr->Refresh(false);
+               viewptr->Update();
+            #endif
+         }
       }
       if (showstatus) {
          statusptr->CheckMouseLocation(IsActive());
@@ -423,6 +486,39 @@ void MainFrame::DisplayPattern()
 
 // -----------------------------------------------------------------------------
 
+bool MainFrame::StepPattern()
+{
+   if (wxGetApp().Poller()->checkevents()) return false;
+   
+   currlayer->algo->step();
+   if (currlayer->autofit) viewptr->FitInView(0);
+   DisplayPattern();
+   
+   /*!!!
+   if (autostop) {
+      int period = currlayer->algo->isPeriodic();
+      if (period > 0) {
+         if (period == 1) {
+            if (currlayer->algo->isEmpty()) {
+               statusptr->DisplayMessage(_("Pattern is empty."));
+            } else {
+               statusptr->DisplayMessage(_("Pattern is stable."));
+            }
+         } else {
+            wxString s;
+            s.Printf(_("Pattern is oscillating (period = %d)."), period);
+            statusptr->DisplayMessage(s);
+         }
+         return false;
+      }
+   }
+   */
+   
+   return true;
+}
+
+// -----------------------------------------------------------------------------
+
 void MainFrame::GeneratePattern()
 {
    if (generating || viewptr->drawingcells || viewptr->waitingforclick) {
@@ -430,8 +526,7 @@ void MainFrame::GeneratePattern()
       return;
    }
 
-   lifealgo* curralgo = currlayer->algo;
-   if (curralgo->isEmpty()) {
+   if (currlayer->algo->isEmpty()) {
       statusptr->ErrorMessage(empty_pattern);
       return;
    }
@@ -446,46 +541,53 @@ void MainFrame::GeneratePattern()
 
    // for DisplayTimingInfo
    begintime = stopwatch->Time();
-   begingen = curralgo->getGeneration().todouble();
+   begingen = currlayer->algo->getGeneration().todouble();
+
+   // for hyperspeed
+   int hypdown = 64;
    
-   generating = true;               // avoid recursion
+   generating = true;               // avoid re-entry
    wxGetApp().PollerReset();
+   
+   #ifdef __WXMSW__
+   wxMenuBar* mbar = GetMenuBar();
+   if (mbar) {
+      // remove any accelerators from the Next Gen and Next Step menu items
+      // so their keyboard shortcuts can be used to stop generating;
+      // this is necessary on Windows because Golly won't see any
+      // key events for a disabled menu item
+      RemoveAccelerator(mbar, ID_NEXT, DO_NEXTGEN);
+      RemoveAccelerator(mbar, ID_STEP, DO_NEXTSTEP);
+   }
+   #endif
    UpdateUserInterface(IsActive());
    
    // only show hashing info while generating, otherwise Mac app can crash
    // after a paste due to hlifealgo::resize() calling lifestatus() which
    // then causes the viewport to be repainted for some inexplicable reason
-   if (currlayer->hash)
-      hlifealgo::setVerbose( currlayer->showhashinfo );
+   lifealgo::setVerbose( currlayer->showhashinfo );
 
-   if (currlayer->warp < 0)
+   if (currlayer->currexpo < 0)
       whentosee = stopwatch->Time() + statusptr->GetCurrentDelay();
-
-   int hypdown = 64;
+   
    while (true) {
-      if (currlayer->warp < 0) {
+      if (currlayer->currexpo < 0) {
          // slow down by only doing one gen every GetCurrentDelay() millisecs
          long currmsec = stopwatch->Time();
          if (currmsec >= whentosee) {
-            if (wxGetApp().Poller()->checkevents()) break;
-            curralgo->step();
-            if (currlayer->autofit) viewptr->FitInView(0);
-            DisplayPattern();
+            if (!StepPattern()) break;
             // add delay to current time rather than currmsec
             whentosee = stopwatch->Time() + statusptr->GetCurrentDelay();
          } else {
             // process events while we wait
             if (wxGetApp().Poller()->checkevents()) break;
-            // don't hog CPU
-            wxMilliSleep(1);     // keep small (ie. <= mindelay)
+            // don't hog CPU but keep sleep duration short (ie. <= mindelay)
+            wxMilliSleep(1);
          }
       } else {
-         // warp >= 0 so only show results every curralgo->getIncrement() gens
-         if (wxGetApp().Poller()->checkevents()) break;
-         curralgo->step();
-         if (currlayer->autofit) viewptr->FitInView(0);
-         DisplayPattern();
-         if (currlayer->hyperspeed && curralgo->hyperCapable()) {
+         // currexpo >= 0 so advance pattern by currlayer->algo->getIncrement() gens
+         if (!StepPattern()) break;
+         if (currlayer->hyperspeed && currlayer->algo->hyperCapable()) {
             hypdown--;
             if (hypdown == 0) {
                hypdown = 64;
@@ -497,11 +599,19 @@ void MainFrame::GeneratePattern()
 
    generating = false;
 
-   if (currlayer->hash) hlifealgo::setVerbose(0);
+   lifealgo::setVerbose(0);
 
    // for DisplayTimingInfo
    endtime = stopwatch->Time();
-   endgen = curralgo->getGeneration().todouble();
+   endgen = currlayer->algo->getGeneration().todouble();
+   
+   #ifdef __WXMSW__
+   if (mbar) {
+      // restore accelerators removed above
+      SetAccelerator(mbar, ID_NEXT, DO_NEXTGEN);
+      SetAccelerator(mbar, ID_STEP, DO_NEXTSTEP);
+   }
+   #endif
    
    // display the final pattern
    if (currlayer->autofit) viewptr->FitInView(0);
@@ -590,19 +700,21 @@ void MainFrame::DoPendingAction(bool restart)
                }
 
             } else {
-               // temporarily pretend the tool/layer bars are not showing
-               // to avoid UpdateToolBar/UpdateLayerBar changing button states
+               // temporarily pretend the tool/layer/edit bars are not showing
+               // to avoid Update[Tool/Layer/Edit]Bar changing button states
                bool saveshowtool = showtool;    showtool = false;
                bool saveshowlayer = showlayer;  showlayer = false;
+               bool saveshowedit = showedit;    showedit = false;
                
                // process the pending command
                cmdevent.SetEventType(wxEVT_COMMAND_MENU_SELECTED);
                cmdevent.SetEventObject(mainptr);
                mainptr->ProcessEvent(cmdevent);
                
-               // restore tool/layer bar flags
+               // restore tool/layer/edit bar flags
                showtool = saveshowtool;
                showlayer = saveshowlayer;
+               showedit = saveshowedit;
                
                if (restart) {
                   // call GeneratePattern again
@@ -616,10 +728,11 @@ void MainFrame::DoPendingAction(bool restart)
    if (draw_pending) {
       draw_pending = false;
 
-      // temporarily pretend the tool/layer bars are not showing
-      // to avoid UpdateToolBar/UpdateLayerBar changing button states
+      // temporarily pretend the tool/layer/edit bars are not showing
+      // to avoid Update[Tool/Layer/Edit]Bar changing button states
       bool saveshowtool = showtool;    showtool = false;
       bool saveshowlayer = showlayer;  showlayer = false;
+      bool saveshowedit = showedit;    showedit = false;
       
       UpdateEverything();
       
@@ -632,9 +745,10 @@ void MainFrame::DoPendingAction(bool restart)
          wxMilliSleep(5);             // don't hog CPU
       }
       
-      // restore tool/layer bar flags
+      // restore tool/layer/edit bar flags
       showtool = saveshowtool;
       showlayer = saveshowlayer;
+      showedit = saveshowedit;
       
       if (restart) {
          // call GeneratePattern again
@@ -646,17 +760,6 @@ void MainFrame::DoPendingAction(bool restart)
 
 // -----------------------------------------------------------------------------
 
-void MainFrame::Stop()
-{
-   if (inscript) {
-      PassKeyToScript(WXK_ESCAPE);
-   } else if (generating) {
-      wxGetApp().PollerInterrupt();
-   }
-}
-
-// -----------------------------------------------------------------------------
-
 void MainFrame::DisplayTimingInfo()
 {
    if (viewptr->waitingforclick) return;
@@ -668,18 +771,43 @@ void MainFrame::DisplayTimingInfo()
       double secs = (double)(endtime - begintime) / 1000.0;
       double gens = endgen - begingen;
       wxString s;
-      s.Printf(_("%g gens in %g secs (%g gens/sec)"), gens, secs, gens / secs);
+      s.Printf(_("%g gens in %g secs (%g gens/sec)."), gens, secs, gens / secs);
       statusptr->DisplayMessage(s);
    }
 }
 
 // -----------------------------------------------------------------------------
 
+void MainFrame::Stop()
+{
+   if (inscript) {
+      PassKeyToScript(WXK_ESCAPE);
+   } else if (generating) {
+      wxGetApp().PollerInterrupt();
+   }
+}
+
+// -----------------------------------------------------------------------------
+
+// this global flag is used to avoid re-entrancy in NextGeneration()
+// due to holding down the space/tab key
+static bool inNextGen = false;
+
 void MainFrame::NextGeneration(bool useinc)
 {
-   if (generating || viewptr->drawingcells || viewptr->waitingforclick) {
-      // don't play sound here because it'll be heard if user holds down tab key
-      // wxBell();
+   if (inNextGen) return;
+   inNextGen = true;
+
+   if (!inscript && generating) {
+      // we must be in GeneratePattern() loop, so abort it
+      Stop();
+      inNextGen = false;
+      return;
+   }
+
+   if (viewptr->drawingcells || viewptr->waitingforclick) {
+      wxBell();
+      inNextGen = false;
       return;
    }
 
@@ -689,21 +817,29 @@ void MainFrame::NextGeneration(bool useinc)
    lifealgo* curralgo = currlayer->algo;
    if (curralgo->isEmpty()) {
       statusptr->ErrorMessage(empty_pattern);
+      inNextGen = false;
       return;
    }
    
    if (!SaveStartingPattern()) {
+      inNextGen = false;
       return;
    }
 
    if (allowundo) {
       if (currlayer->stayclean) {
-         // script has called run/step after a command (eg. new)
-         // has set stayclean true by calling MarkLayerClean
+         // script has called run/step after a new/open command has set
+         // stayclean true by calling MarkLayerClean
          if (curralgo->getGeneration() == currlayer->startgen) {
             // starting pattern has just been saved so we need to remember
             // this gen change in case user does a Reset after script ends
             // (RememberGenFinish will be called at the end of RunScript)
+            if (currlayer->undoredo->savegenchanges) {
+               // script must have called reset command, so we need to call
+               // RememberGenFinish to match earlier RememberGenStart
+               currlayer->undoredo->savegenchanges = false;
+               currlayer->undoredo->RememberGenFinish();
+            }
             currlayer->undoredo->RememberGenStart();
          }
       } else {
@@ -717,11 +853,11 @@ void MainFrame::NextGeneration(bool useinc)
       }
    }
 
-   // curralgo->step() calls checkevents so set generating flag to avoid recursion
+   // curralgo->step() calls checkevents() so set generating flag
    generating = true;
 
    // only show hashing info while generating
-   if (currlayer->hash) hlifealgo::setVerbose( currlayer->showhashinfo );
+   lifealgo::setVerbose( currlayer->showhashinfo );
    
    // avoid doing some things if NextGeneration is called from a script;
    // ie. by a run/step command
@@ -747,7 +883,7 @@ void MainFrame::NextGeneration(bool useinc)
 
    generating = false;
 
-   if (currlayer->hash) hlifealgo::setVerbose(0);
+   lifealgo::setVerbose(0);
    
    if (!inscript) {
       // autofit is only used when doing many gens
@@ -756,11 +892,15 @@ void MainFrame::NextGeneration(bool useinc)
       UpdateEverything();
    }
 
+   // we must call RememberGenFinish BEFORE processing any pending command
    if (allowundo && !currlayer->stayclean)
       currlayer->undoredo->RememberGenFinish();
    
+   // process any pending command seen via checkevents() in curralgo->step()
    if (!inscript)
-      DoPendingAction(false);     // true means don't restart generating loop
+      DoPendingAction(false);     // false means don't restart generating loop
+   
+   inNextGen = false;
 }
 
 // -----------------------------------------------------------------------------
@@ -768,9 +908,10 @@ void MainFrame::NextGeneration(bool useinc)
 void MainFrame::ToggleAutoFit()
 {
    currlayer->autofit = !currlayer->autofit;
+   
    // we only use autofit when generating; that's why the Auto Fit item
    // is in the Control menu and not in the View menu
-   if (currlayer->autofit && generating) {
+   if (generating && currlayer->autofit) {
       viewptr->FitInView(0);
       UpdateEverything();
    }
@@ -778,20 +919,152 @@ void MainFrame::ToggleAutoFit()
 
 // -----------------------------------------------------------------------------
 
-void MainFrame::ToggleHashing()
+void MainFrame::ToggleHyperspeed()
 {
-   if ( global_liferules.hasB0notS8 && !currlayer->hash ) {
-      statusptr->ErrorMessage(_("Hashing cannot be used with a B0-not-S8 rule."));
+   currlayer->hyperspeed = !currlayer->hyperspeed;
+}
+
+// -----------------------------------------------------------------------------
+
+void MainFrame::ToggleHashInfo()
+{
+   currlayer->showhashinfo = !currlayer->showhashinfo;
+   
+   // only show hashing info while generating
+   if (generating) lifealgo::setVerbose( currlayer->showhashinfo );
+}
+
+// -----------------------------------------------------------------------------
+
+void MainFrame::ReduceCellStates(int newmaxstate)
+{
+   // check current pattern and reduce any cell states > newmaxstate
+   bool patternchanged = false;
+   bool savechanges = allowundo && !currlayer->stayclean;
+
+   // check if current pattern is too big to use nextcell/setcell
+   bigint top, left, bottom, right;
+   currlayer->algo->findedges(&top, &left, &bottom, &right);
+   if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
+      statusptr->ErrorMessage(_("Pattern too big to check (outside +/- 10^9 boundary)."));
+      return;
+   }
+   
+   int itop = top.toint();
+   int ileft = left.toint();
+   int ibottom = bottom.toint();
+   int iright = right.toint();
+   int ht = ibottom - itop + 1;
+   int cx, cy;
+
+   // for showing accurate progress we need to add pattern height to pop count
+   // in case this is a huge pattern with many blank rows
+   double maxcount = currlayer->algo->getPopulation().todouble() + ht;
+   double accumcount = 0;
+   int currcount = 0;
+   bool abort = false;
+   int v = 0;
+   BeginProgress(_("Checking cell states"));
+   
+   lifealgo* curralgo = currlayer->algo;
+   for ( cy=itop; cy<=ibottom; cy++ ) {
+      currcount++;
+      for ( cx=ileft; cx<=iright; cx++ ) {
+         int skip = curralgo->nextcell(cx, cy, v);
+         if (skip >= 0) {
+            // found next live cell in this row
+            cx += skip;
+            if (v > newmaxstate) {
+               // reduce cell's current state to largest state
+               if (savechanges) currlayer->undoredo->SaveCellChange(cx, cy, v, newmaxstate);
+               curralgo->setcell(cx, cy, newmaxstate);
+               patternchanged = true;
+            }
+            currcount++;
+         } else {
+            cx = iright;  // done this row
+         }
+         if (currcount > 1024) {
+            accumcount += currcount;
+            currcount = 0;
+            abort = AbortProgress(accumcount / maxcount, wxEmptyString);
+            if (abort) break;
+         }
+      }
+      if (abort) break;
+   }
+   
+   curralgo->endofpattern();
+   EndProgress();
+
+   if (patternchanged) {
+      statusptr->ErrorMessage(_("Pattern has changed (new rule has fewer states)."));
+   }
+}
+
+// -----------------------------------------------------------------------------
+
+void MainFrame::ShowRuleDialog()
+{
+   if (inscript || viewptr->waitingforclick) return;
+
+   if (generating) {
+      // terminate generating loop and set command_pending flag
+      Stop();
+      command_pending = true;
+      cmdevent.SetId(ID_SETRULE);
       return;
    }
 
-   // check if current pattern is too big to use getcell/setcell
+   algo_type oldalgo = currlayer->algtype;
+   wxString oldrule = wxString(currlayer->algo->getrule(), wxConvLocal);
+   int oldmaxstate = currlayer->algo->NumCellStates() - 1;
+
+   if (ChangeRule()) {
+      // if ChangeAlgorithm was called then we're done
+      if (currlayer->algtype != oldalgo) {
+         // except we have to call UpdateEverything here now that the main window is active
+         UpdateEverything();
+         return;
+      }
+      
+      // show new rule in window title (but don't change file name);
+      // even if the rule didn't change we still need to do this because
+      // the user might have simply added/deleted a named rule
+      SetWindowTitle(wxEmptyString);
+      
+      // check if rule actually changed
+      wxString newrule = wxString(currlayer->algo->getrule(), wxConvLocal);
+      if (oldrule != newrule) {
+         // rule change might have changed the number of cell states;
+         // if there are fewer states then pattern might change
+         int newmaxstate = currlayer->algo->NumCellStates() - 1;
+         if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
+            ReduceCellStates(newmaxstate);
+         }
+         
+         // pattern might have changed or new rule might have changed colors
+         UpdateEverything();
+         
+         if (allowundo) {
+            currlayer->undoredo->RememberRuleChange(oldrule);
+         }
+      }
+   }
+}
+
+// -----------------------------------------------------------------------------
+
+void MainFrame::ChangeAlgorithm(algo_type newalgotype, const wxString& newrule, bool inundoredo)
+{
+   if (newalgotype == currlayer->algtype) return;
+
+   // check if current pattern is too big to use nextcell/setcell
    bigint top, left, bottom, right;
    if ( !currlayer->algo->isEmpty() ) {
       currlayer->algo->findedges(&top, &left, &bottom, &right);
       if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
          statusptr->ErrorMessage(_("Pattern cannot be converted (outside +/- 10^9 boundary)."));
-         // ask user if they want to continue anyway???
          return;
       }
    }
@@ -800,31 +1073,57 @@ void MainFrame::ToggleHashing()
       // terminate generating loop and set command_pending flag
       Stop();
       command_pending = true;
-      cmdevent.SetId(ID_HASH);
+      cmdevent.SetId(ID_ALGO0 + newalgotype);
       return;
    }
-   
-   if (allowundo && !currlayer->stayclean && inscript) {
-      // note that we must save pending gen changes BEFORE changing hash flag
+
+   // save changes if undo/redo is enabled and script isn't constructing a pattern
+   // and we're not undoing/redoing an earlier algo change
+   bool savechanges = allowundo && !currlayer->stayclean && !inundoredo;
+   if (savechanges && inscript) {
+      // note that we must save pending gen changes BEFORE changing algo type
       // otherwise temporary files won't be the correct type (mc or rle)
       SavePendingChanges();
    }
+   
+   bool rulechanged = false;
+   wxString oldrule = wxString(currlayer->algo->getrule(), wxConvLocal);
 
-   // toggle hashing option and update status bar immediately
-   currlayer->hash = !currlayer->hash;
-   currlayer->warp = 0;
+   // change algorithm type, reset step size, and update status bar immediately
+   algo_type oldalgo = currlayer->algtype;
+   currlayer->algtype = newalgotype;
+   currlayer->currbase = algoinfo[newalgotype]->defbase;
+   currlayer->currexpo = 0;
    UpdateStatus();
 
-   // create a new universe of the right flavor
-   lifealgo* newalgo = CreateNewUniverse(currlayer->hash);
+   // create a new universe of the requested flavor
+   lifealgo* newalgo = CreateNewUniverse(newalgotype);
    
-   // even though universes share a global rule table we still need to call setrule
-   // due to internal differences in the handling of Wolfram rules
-   newalgo->setrule( (char*)currlayer->algo->getrule() );
+   if (inundoredo) {
+      // switch to given newrule (no error should occur)
+      const char* err = newalgo->setrule( newrule.mb_str(wxConvLocal) );
+      if (err) Warning(_("Bug detected in ChangeAlgorithm!"));
+   } else {
+      const char* err;
+      if (newrule.IsEmpty()) {
+         // try to use same rule
+         err = newalgo->setrule( currlayer->algo->getrule() );
+      } else {
+         // switch to newrule (ChangeRule has called ChangeAlgorithm)
+         err = newalgo->setrule( newrule.mb_str(wxConvLocal) );
+         rulechanged = true;
+      }
+      if (err) {
+         // switch to new algo's default rule
+         newalgo->setrule( newalgo->DefaultRule() );
+         rulechanged = true;
+      }
+   }
    
    // set same gen count
    newalgo->setGeneration( currlayer->algo->getGeneration() );
 
+   bool patternchanged = false;
    if ( !currlayer->algo->isEmpty() ) {
       // copy pattern in current universe to new universe
       int itop = top.toint();
@@ -840,17 +1139,28 @@ void MainFrame::ToggleHashing()
       double accumcount = 0;
       int currcount = 0;
       bool abort = false;
+      int v = 0;
       BeginProgress(_("Converting pattern"));
-   
+      
       lifealgo* curralgo = currlayer->algo;
+      
+      // need to check for state change if new algo has fewer states than old algo
+      int newmaxstate = newalgo->NumCellStates() - 1;
+   
       for ( cy=itop; cy<=ibottom; cy++ ) {
          currcount++;
          for ( cx=ileft; cx<=iright; cx++ ) {
-            int skip = curralgo->nextcell(cx, cy);
+            int skip = curralgo->nextcell(cx, cy, v);
             if (skip >= 0) {
                // found next live cell in this row
                cx += skip;
-               newalgo->setcell(cx, cy, 1);
+               if (v > newmaxstate) {
+                  // reduce v to largest state in new algo
+                  if (savechanges) currlayer->undoredo->SaveCellChange(cx, cy, v, newmaxstate);
+                  v = newmaxstate;
+                  patternchanged = true;
+               }
+               newalgo->setcell(cx, cy, v);
                currcount++;
             } else {
                cx = iright;  // done this row
@@ -873,55 +1183,49 @@ void MainFrame::ToggleHashing()
    delete currlayer->algo;
    currlayer->algo = newalgo;   
    SetGenIncrement();
-   UpdateEverything();
    
-   if (allowundo && !currlayer->stayclean)
-      currlayer->undoredo->RememberAlgoChange();
-}
+   // switch to default colors for new algo+rule
+   UpdateLayerColors();
 
-// -----------------------------------------------------------------------------
-
-void MainFrame::ToggleHyperspeed()
-{
-   currlayer->hyperspeed = !currlayer->hyperspeed;
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::ToggleHashInfo()
-{
-   currlayer->showhashinfo = !currlayer->showhashinfo;
-   
-   // only show hashing info while generating
-   if (generating && currlayer->hash)
-      hlifealgo::setVerbose( currlayer->showhashinfo );
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::SetWarp(int newwarp)
-{
-   currlayer->warp = newwarp;
-   if (currlayer->warp < minwarp) currlayer->warp = minwarp;
-   SetGenIncrement();
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::ShowRuleDialog()
-{
-   if (inscript || viewptr->waitingforclick) return;
-
-   if (generating) {
-      // terminate generating loop and set command_pending flag
-      Stop();
-      command_pending = true;
-      cmdevent.SetId(ID_RULE);
-      return;
+   if (!inundoredo) {
+      if (rulechanged) {
+         // show new rule in window title (but don't change file name)
+         SetWindowTitle(wxEmptyString);
+         
+         // if pattern exists and is at starting gen then set savestart true
+         // so that SaveStartingPattern will save pattern to suitable file
+         // (and thus ResetPattern will work correctly)
+         if ( currlayer->algo->getGeneration() == currlayer->startgen &&
+              !currlayer->algo->isEmpty() ) {
+            currlayer->savestart = true;
+         }
+         
+         if (newrule.IsEmpty()) {
+            if (patternchanged) {
+               statusptr->ErrorMessage(_("Rule has changed and pattern has changed (new algorithm has fewer states)."));
+            } else {
+               // don't beep
+               statusptr->DisplayMessage(_("Rule has changed."));
+            }
+         } else {
+            // ChangeRule called ChangeAlgorithm
+            if (patternchanged) {
+               statusptr->ErrorMessage(_("Algorithm has changed and pattern has changed (new algorithm has fewer states)."));
+            } else {
+               // don't beep
+               statusptr->DisplayMessage(_("Algorithm has changed."));
+            }
+         }
+      } else if (patternchanged) {
+         statusptr->ErrorMessage(_("Pattern has changed (new algorithm has fewer states)."));
+      }
+      
+      if (!inscript) {
+         UpdateEverything();
+      }
    }
 
-   if (ChangeRule()) {
-      // show new rule in window title (file name doesn't change)
-      SetWindowTitle(wxEmptyString);
+   if (savechanges) {
+      currlayer->undoredo->RememberAlgoChange(oldalgo, oldrule);
    }
 }
