@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "wxview.h"        // for viewptr->...
 #include "wxmain.h"        // for mainptr->...
 #include "wxscript.h"      // for inscript
+#include "wxalgos.h"       // for algoinfo
 #include "wxlayer.h"       // for currlayer
 #include "wxstatus.h"
 
@@ -75,6 +76,8 @@ void StatusBar::ClearMessage()
 {
    if (inscript) return;                     // let script control messages
    if (viewptr->waitingforclick) return;     // don't clobber message
+   if (statusmsg.IsEmpty()) return;          // no need to clear message
+   
    statusmsg.Clear();
    if (statusht > 0) {
       int wd, ht;
@@ -84,8 +87,8 @@ void StatusBar::ClearMessage()
          wxRect r = wxRect( wxPoint(0,statusht-BOTGAP+DESCHT-LINEHT),
                             wxPoint(wd-1,ht-1) );
          Refresh(false, &r);
-         // don't call Update() otherwise Win/X11 users see blue & yellow bands
-         // when toggling hashing option
+         // nicer not to call Update() here otherwise users can see different
+         // colored bands in status bar when changing algos
       }
    }
 }
@@ -237,7 +240,7 @@ wxString StatusBar::Stringify(const bigint& b)
 
 int StatusBar::GetCurrentDelay()
 {
-   int gendelay = mindelay * (1 << (-currlayer->warp - 1));
+   int gendelay = mindelay * (1 << (-currlayer->currexpo - 1));
    if (gendelay > maxdelay) gendelay = maxdelay;
    return gendelay;
 }
@@ -251,7 +254,7 @@ void StatusBar::DrawStatusBar(wxDC& dc, wxRect& updaterect)
    if (wd < 1 || ht < 1) return;
 
    wxRect r = wxRect(0, 0, wd, ht);
-   FillRect(dc, r, currlayer->hash ? *hlifebrush : *qlifebrush);
+   FillRect(dc, r, *(algoinfo[currlayer->algtype]->statusbrush));
 
    #if defined(__WXMSW__)
       // draw gray lines at top and left edges
@@ -295,7 +298,13 @@ void StatusBar::DrawStatusBar(wxDC& dc, wxRect& updaterect)
          if (viewptr->nopattupdate) {
             strbuf += _("0");
          } else {
-            strbuf += wxString(currlayer->algo->getPopulation().tostring(), wxConvLocal);
+            bigint popcount = currlayer->algo->getPopulation();
+            if (popcount.sign() < 0) {
+               // getPopulation returns -1 if it can't be calculated
+               strbuf += _("?");
+            } else {
+               strbuf += wxString(popcount.tostring(), wxConvLocal);
+            }
          }
          DisplayText(dc, strbuf, h_gen, POPLINE);
          
@@ -307,16 +316,12 @@ void StatusBar::DrawStatusBar(wxDC& dc, wxRect& updaterect)
          }
          DisplayText(dc, strbuf, h_gen, SCALELINE);
          
-         if (currlayer->warp < 0) {
+         if (currlayer->currexpo < 0) {
             // show delay in secs
             strbuf.Printf(_("Delay = %gs"), (double)GetCurrentDelay() / 1000.0);
          } else {
-            // no real need to show step as an exact number???
-            if (currlayer->hash) {
-               strbuf.Printf(_("Step = %d^%d"), hbasestep, currlayer->warp);
-            } else {
-               strbuf.Printf(_("Step = %d^%d"), qbasestep, currlayer->warp);
-            }
+            // no real need to show step as an exact number
+            strbuf.Printf(_("Step = %d^%d"), currlayer->currbase, currlayer->currexpo);
          }
          DisplayText(dc, strbuf, h_gen, STEPLINE);
       }
@@ -355,7 +360,13 @@ void StatusBar::DrawStatusBar(wxDC& dc, wxRect& updaterect)
          if (viewptr->nopattupdate) {
             strbuf += _("0");
          } else {
-            strbuf += Stringify(currlayer->algo->getPopulation());
+            bigint popcount = currlayer->algo->getPopulation();
+            if (popcount.sign() < 0) {
+               // getPopulation returns -1 if it can't be calculated
+               strbuf += _("?");
+            } else {
+               strbuf += Stringify(popcount);
+            }
          }
          DisplayText(dc, strbuf, h_pop, BASELINE1);
       
@@ -366,15 +377,11 @@ void StatusBar::DrawStatusBar(wxDC& dc, wxRect& updaterect)
          }
          DisplayText(dc, strbuf, h_scale, BASELINE1);
          
-         if (currlayer->warp < 0) {
+         if (currlayer->currexpo < 0) {
             // show delay in secs
             strbuf.Printf(_("Delay=%gs"), (double)GetCurrentDelay() / 1000.0);
          } else {
-            if (currlayer->hash) {
-               strbuf.Printf(_("Step=%d^%d"), hbasestep, currlayer->warp);
-            } else {
-               strbuf.Printf(_("Step=%d^%d"), qbasestep, currlayer->warp);
-            }
+            strbuf.Printf(_("Step=%d^%d"), currlayer->currbase, currlayer->currexpo);
          }
          DisplayText(dc, strbuf, h_step, BASELINE1);
       }
@@ -430,7 +437,7 @@ void StatusBar::OnPaint(wxPaintEvent& WXUNUSED(event))
       if (ht < 1) ht = 1;
       if (wd != statbitmapwd || ht != statbitmapht) {
          // need to create a new bitmap for status bar
-         if (statbitmap) delete statbitmap;
+         delete statbitmap;
          statbitmap = new wxBitmap(wd, ht);
          statbitmapwd = wd;
          statbitmapht = ht;
@@ -490,9 +497,11 @@ void StatusBar::OnMouseDown(wxMouseEvent& event)
       }
 
    } else if ( ClickInStepBox(event.GetX(), event.GetY()) ) {
-      if (currlayer->warp != 0) {
-         // reset step to 1 gen
-         mainptr->SetWarp(0);
+      if (currlayer->currbase != algoinfo[currlayer->algtype]->defbase ||
+          currlayer->currexpo != 0) {
+         // reset base step to default value and step exponent to 0
+         currlayer->currbase = algoinfo[currlayer->algtype]->defbase;
+         mainptr->SetStepExponent(0);
          // update status bar
          Refresh(false);
          Update();
@@ -573,12 +582,14 @@ StatusBar::StatusBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int h
    statbitmap = NULL;
    statbitmapwd = -1;
    statbitmapht = -1;
+   
+   statusmsg.Clear();
 }
 
 // -----------------------------------------------------------------------------
 
 StatusBar::~StatusBar()
 {
-   if (statusfont) delete statusfont;
-   if (statbitmap) delete statbitmap;
+   delete statusfont;
+   delete statbitmap;
 }
