@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,10 +35,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #include "wxgolly.h"       // for mainptr, viewptr
 #include "wxmain.h"        // for mainptr->...
-#include "wxedit.h"        // for Selection
+#include "wxselect.h"      // for Selection
 #include "wxview.h"        // for viewptr->...
 #include "wxutils.h"       // for Warning, Fatal
 #include "wxscript.h"      // for inscript
+#include "wxalgos.h"       // for algo_type
 #include "wxlayer.h"       // for currlayer, numclones, MarkLayerDirty, etc
 #include "wxprefs.h"       // for allowundo, GetAccelerator, etc
 #include "wxundo.h"
@@ -46,8 +47,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // -----------------------------------------------------------------------------
 
 const wxString lack_of_memory = _("Due to lack of memory, some changes can't be undone!");
-const wxString temp_prefix = _("golly_undo_");
 const wxString to_gen = _("to Gen ");
+const wxString temp_prefix = wxT("golly_undo_");
 
 // -----------------------------------------------------------------------------
 
@@ -71,7 +72,7 @@ typedef enum {
    genchange,           // pattern was generated
    setgen,              // generation count was changed
    rulechange,          // rule was changed
-   algochange,          // algorithm was changed (ie. hashing was toggled)
+   algochange,          // algorithm was changed
    scriptstart,         // later changes were made by script
    scriptfinish         // earlier changes were made by script
 } change_type;
@@ -85,14 +86,17 @@ public:
    // do the undo/redo; if it returns false (eg. user has aborted a lengthy
    // rotate/flip operation) then cancel the undo/redo
 
+   void ChangeCells(bool undo);
+   // change cell states using cellinfo
+
    change_type changeid;                  // specifies the type of change
    wxString suffix;                       // action string for Undo/Redo item
    bool olddirty;                         // layer's dirty state before change
    bool newdirty;                         // layer's dirty state after change
    
    // cellstates info
-   int* cellcoords;                       // array of x,y coordinates (2 ints per cell)
-   unsigned int cellcount;                // number of cells in array
+   cell_change* cellinfo;                 // dynamic array of cell changes
+   unsigned int cellcount;                // number of cell changes in array
    
    // rotatecw/rotateacw/selchange info
    Selection oldsel, newsel;              // old and new selections
@@ -102,14 +106,14 @@ public:
    bigint oldgen, newgen;                 // old and new generation counts
    bigint oldx, oldy, newx, newy;         // old and new positions
    int oldmag, newmag;                    // old and new scales
-   int oldwarp, newwarp;                  // old and new speeds
+   int oldbase, newbase;                  // old and new base steps
+   int oldexpo, newexpo;                  // old and new step exponents
    bool scriptgen;                        // gen change was done by script?
    // also uses oldsel, newsel
    
    // setgen info
    bigint oldstartgen, newstartgen;       // old and new startgen values
    bool oldsave, newsave;                 // old and new savestart states
-   bool oldhash, newhash;                 // old and new hash states
    wxString oldtempstart, newtempstart;   // old and new tempstart paths
    wxString oldstartfile, newstartfile;   // old and new startfile paths
    wxString oldcurrfile, newcurrfile;     // old and new currfile paths
@@ -118,8 +122,10 @@ public:
    // also uses oldgen, newgen
    // and oldrule, newrule
    // and oldx, oldy, newx, newy, oldmag, newmag
-   // and oldwarp, newwarp
+   // and oldbase, newbase
+   // and oldexpo, newexpo
    // and oldsel, newsel
+   // and oldalgo, newalgo
    
    // namechange info
    wxString oldname, newname;             // old and new layer names
@@ -129,6 +135,10 @@ public:
 
    // rulechange info
    wxString oldrule, newrule;             // old and new rules
+   
+   // algochange info
+   algo_type oldalgo, newalgo;            // old and new algorithm types
+   // also uses oldrule, newrule
 };
 
 // -----------------------------------------------------------------------------
@@ -136,7 +146,8 @@ public:
 ChangeNode::ChangeNode(change_type id)
 {
    changeid = id;
-   cellcoords = NULL;
+   cellinfo = NULL;
+   cellcount = 0;
    oldfile = wxEmptyString;
    newfile = wxEmptyString;
    oldtempstart = wxEmptyString;
@@ -147,7 +158,7 @@ ChangeNode::ChangeNode(change_type id)
 
 ChangeNode::~ChangeNode()
 {
-   if (cellcoords) free(cellcoords);
+   if (cellinfo) free(cellinfo);
    
    if (!oldfile.IsEmpty() && wxFileExists(oldfile)) {
       wxRemoveFile(oldfile);
@@ -174,20 +185,35 @@ ChangeNode::~ChangeNode()
 
 // -----------------------------------------------------------------------------
 
+void ChangeNode::ChangeCells(bool undo)
+{
+   // change state of cell(s) stored in cellinfo array
+   if (undo) {
+      // we must undo the cell changes in reverse order in case
+      // a script has changed the same cell more than once
+      unsigned int i = cellcount;
+      while (i > 0) {
+         i--;
+         currlayer->algo->setcell(cellinfo[i].x, cellinfo[i].y, cellinfo[i].oldstate);
+      }
+   } else {
+      unsigned int i = 0;
+      while (i < cellcount) {
+         currlayer->algo->setcell(cellinfo[i].x, cellinfo[i].y, cellinfo[i].newstate);
+         i++;
+      }
+   }
+   if (cellcount > 0) currlayer->algo->endofpattern();
+}
+
+// -----------------------------------------------------------------------------
+
 bool ChangeNode::DoChange(bool undo)
 {
    switch (changeid) {
       case cellstates:
-         // change state of cell(s) stored in cellcoords array
-         {
-            unsigned int i = 0;
-            while (i < cellcount * 2) {
-               int x = cellcoords[i++];
-               int y = cellcoords[i++];
-               int state = currlayer->algo->getcell(x, y);
-               currlayer->algo->setcell(x, y, 1 - state);
-            }
-            currlayer->algo->endofpattern();
+         if (cellcount > 0) {
+            ChangeCells(undo);
             mainptr->UpdatePatternAndStatus();
          }
          break;
@@ -208,16 +234,8 @@ bool ChangeNode::DoChange(bool undo)
 
       case rotatecw:
       case rotateacw:
-         // change state of cell(s) stored in cellcoords array
-         {
-            unsigned int i = 0;
-            while (i < cellcount * 2) {
-               int x = cellcoords[i++];
-               int y = cellcoords[i++];
-               int state = currlayer->algo->getcell(x, y);
-               currlayer->algo->setcell(x, y, 1 - state);
-            }
-            currlayer->algo->endofpattern();
+         if (cellcount > 0) {
+            ChangeCells(undo);
          }
          // rotate selection edges
          if (undo) {
@@ -242,10 +260,10 @@ bool ChangeNode::DoChange(bool undo)
       case genchange:
          if (undo) {
             currlayer->currsel = oldsel;
-            mainptr->RestorePattern(oldgen, oldfile, oldx, oldy, oldmag, oldwarp);
+            mainptr->RestorePattern(oldgen, oldfile, oldx, oldy, oldmag, oldbase, oldexpo);
          } else {
             currlayer->currsel = newsel;
-            mainptr->RestorePattern(newgen, newfile, newx, newy, newmag, newwarp);
+            mainptr->RestorePattern(newgen, newfile, newx, newy, newmag, newbase, newexpo);
          }
          break;
       
@@ -259,12 +277,13 @@ bool ChangeNode::DoChange(bool undo)
             currlayer->currfile = oldcurrfile;
             if (oldtempstart != newtempstart) {
                currlayer->startdirty = olddirty;
-               currlayer->starthash = oldhash;
+               currlayer->startalgo = oldalgo;
                currlayer->startrule = oldrule;
                currlayer->startx = oldx;
                currlayer->starty = oldy;
                currlayer->startmag = oldmag;
-               currlayer->startwarp = oldwarp;
+               currlayer->startbase = oldbase;
+               currlayer->startexpo = oldexpo;
                currlayer->startsel = oldsel;
                currlayer->startname = oldname;
                if (currlayer->cloneid > 0) {
@@ -285,12 +304,13 @@ bool ChangeNode::DoChange(bool undo)
             currlayer->currfile = newcurrfile;
             if (oldtempstart != newtempstart) {
                currlayer->startdirty = newdirty;
-               currlayer->starthash = newhash;
+               currlayer->startalgo = newalgo;
                currlayer->startrule = newrule;
                currlayer->startx = newx;
                currlayer->starty = newy;
                currlayer->startmag = newmag;
-               currlayer->startwarp = newwarp;
+               currlayer->startbase = newbase;
+               currlayer->startexpo = newexpo;
                currlayer->startsel = newsel;
                currlayer->startname = newname;
                if (currlayer->cloneid > 0) {
@@ -335,20 +355,36 @@ bool ChangeNode::DoChange(bool undo)
          break;
 
       case rulechange:
-         if (undo) {
-            currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
-         } else {
-            currlayer->algo->setrule( newrule.mb_str(wxConvLocal) );
+         {
+            if (undo) {
+               currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+            } else {
+               currlayer->algo->setrule( newrule.mb_str(wxConvLocal) );
+            }
+            // show new rule in window title (file name doesn't change)
+            mainptr->SetWindowTitle(wxEmptyString);
+            if (cellcount > 0) {
+               ChangeCells(undo);
+            }
+            // switch to default colors for new rule
+            UpdateLayerColors();
          }
-         // show new rule in window title (file name doesn't change)
-         mainptr->SetWindowTitle(wxEmptyString);
+         mainptr->UpdateEverything();
          break;
 
       case algochange:
-         // temporarily reset allowundo so ToggleHashing won't call RememberAlgoChange
-         allowundo = false;
-         mainptr->ToggleHashing();
-         allowundo = true;
+         // pass in true so ChangeAlgorithm won't call RememberAlgoChange
+         if (undo) {
+            mainptr->ChangeAlgorithm(oldalgo, oldrule, true);
+         } else {
+            mainptr->ChangeAlgorithm(newalgo, newrule, true);
+         }
+         // show new rule in window title (file name doesn't change)
+         mainptr->SetWindowTitle(wxEmptyString);
+         if (cellcount > 0) {
+            ChangeCells(undo);
+         }
+         mainptr->UpdateEverything();
          break;
       
       case scriptstart:
@@ -364,8 +400,8 @@ bool ChangeNode::DoChange(bool undo)
 
 UndoRedo::UndoRedo()
 {
-   intcount = 0;                 // for 1st SaveCellChange
-   maxcount = 0;                 // ditto
+   numchanges = 0;               // for 1st SaveCellChange
+   maxchanges = 0;               // ditto
    badalloc = false;             // true if malloc/realloc fails
    cellarray = NULL;             // play safe
    savecellchanges = false;      // no script cell changes are pending
@@ -388,13 +424,13 @@ UndoRedo::~UndoRedo()
 
 // -----------------------------------------------------------------------------
 
-void UndoRedo::SaveCellChange(int x, int y)
+void UndoRedo::SaveCellChange(int x, int y, int oldstate, int newstate)
 {
-   if (intcount == maxcount) {
-      if (intcount == 0) {
-         // initially allocate room for 1 cell (x and y coords)
-         maxcount = 2;
-         cellarray = (int*) malloc(maxcount * sizeof(int));
+   if (numchanges == maxchanges) {
+      if (numchanges == 0) {
+         // initially allocate room for 1 cell change
+         maxchanges = 1;
+         cellarray = (cell_change*) malloc(maxchanges * sizeof(cell_change));
          if (cellarray == NULL) {
             badalloc = true;
             return;
@@ -402,32 +438,37 @@ void UndoRedo::SaveCellChange(int x, int y)
          // ~ChangeNode or ForgetCellChanges will free cellarray
       } else {
          // double size of cellarray
-         int* newptr = (int*) realloc(cellarray, maxcount * 2 * sizeof(int));
+         cell_change* newptr =
+            (cell_change*) realloc(cellarray, maxchanges * 2 * sizeof(cell_change));
          if (newptr == NULL) {
             badalloc = true;
             return;
          }
          cellarray = newptr;
-         maxcount *= 2;
+         maxchanges *= 2;
       }
    }
    
-   cellarray[intcount++] = x;
-   cellarray[intcount++] = y;
+   cellarray[numchanges].x = x;
+   cellarray[numchanges].y = y;
+   cellarray[numchanges].oldstate = oldstate;
+   cellarray[numchanges].newstate = newstate;
+   
+   numchanges++;
 }
 
 // -----------------------------------------------------------------------------
 
 void UndoRedo::ForgetCellChanges()
 {
-   if (intcount > 0) {
+   if (numchanges > 0) {
       if (cellarray) {
          free(cellarray);
       } else {
          Warning(_("Bug detected in ForgetCellChanges!"));
       }
-      intcount = 0;        // reset for next SaveCellChange
-      maxcount = 0;        // ditto
+      numchanges = 0;      // reset for next SaveCellChange
+      maxchanges = 0;      // ditto
       badalloc = false;
    }
 }
@@ -436,10 +477,11 @@ void UndoRedo::ForgetCellChanges()
 
 bool UndoRedo::RememberCellChanges(const wxString& action, bool olddirty)
 {
-   if (intcount > 0) {
-      if (intcount < maxcount) {
+   if (numchanges > 0) {
+      if (numchanges < maxchanges) {
          // reduce size of cellarray
-         int* newptr = (int*) realloc(cellarray, intcount * sizeof(int));
+         cell_change* newptr =
+            (cell_change*) realloc(cellarray, numchanges * sizeof(cell_change));
          if (newptr != NULL) cellarray = newptr;
          // in the unlikely event that newptr is NULL, cellarray should
          // still point to valid data
@@ -454,8 +496,8 @@ bool UndoRedo::RememberCellChanges(const wxString& action, bool olddirty)
       if (change == NULL) Fatal(_("Failed to create cellstates node!"));
       
       change->suffix = action;
-      change->cellcoords = cellarray;
-      change->cellcount = intcount / 2;
+      change->cellinfo = cellarray;
+      change->cellcount = numchanges;
       change->olddirty = olddirty;
       change->newdirty = true;
       
@@ -464,8 +506,8 @@ bool UndoRedo::RememberCellChanges(const wxString& action, bool olddirty)
       // update Undo item in Edit menu
       UpdateUndoItem(change->suffix);
       
-      intcount = 0;        // reset for next SaveCellChange
-      maxcount = 0;        // ditto
+      numchanges = 0;      // reset for next SaveCellChange
+      maxchanges = 0;      // ditto
 
       if (badalloc) {
          Warning(lack_of_memory);
@@ -539,19 +581,20 @@ void UndoRedo::RememberRotation(bool clockwise, Selection& oldsel, Selection& ne
    change->olddirty = olddirty;
    change->newdirty = true;
 
-   // if intcount == 0 we still need to rotate selection edges
-   if (intcount > 0) {
-      if (intcount < maxcount) {
+   // if numchanges == 0 we still need to rotate selection edges
+   if (numchanges > 0) {
+      if (numchanges < maxchanges) {
          // reduce size of cellarray
-         int* newptr = (int*) realloc(cellarray, intcount * sizeof(int));
+         cell_change* newptr =
+            (cell_change*) realloc(cellarray, numchanges * sizeof(cell_change));
          if (newptr != NULL) cellarray = newptr;
       }
 
-      change->cellcoords = cellarray;
-      change->cellcount = intcount / 2;
+      change->cellinfo = cellarray;
+      change->cellcount = numchanges;
       
-      intcount = 0;        // reset for next SaveCellChange
-      maxcount = 0;        // ditto
+      numchanges = 0;      // reset for next SaveCellChange
+      maxchanges = 0;      // ditto
       if (badalloc) {
          Warning(lack_of_memory);
          badalloc = false;
@@ -606,7 +649,8 @@ void UndoRedo::RememberSelection(const wxString& action)
 void UndoRedo::SaveCurrentPattern(const wxString& tempfile)
 {
    const char* err = NULL;
-   if ( currlayer->hash ) {
+   //!!! need lifealgo::CanWriteFormat(MC_format) method???
+   if ( currlayer->algo->hyperCapable() ) {
       // save hlife pattern in a macrocell file
       err = mainptr->WritePattern(tempfile, MC_format, 0, 0, 0, 0);
    } else {
@@ -633,7 +677,7 @@ void UndoRedo::RememberGenStart()
    startcount++;
    if (startcount > 1) {
       // return immediately and ignore next RememberGenFinish call;
-      // this can happen in wxGTK app if user holds down space bar
+      // this can happen in Linux app if user holds down space bar
       return;
    }
 
@@ -649,7 +693,8 @@ void UndoRedo::RememberGenStart()
    prevsel = currlayer->currsel;
    viewptr->GetPos(prevx, prevy);
    prevmag = viewptr->GetMag();
-   prevwarp = currlayer->warp;
+   prevbase = currlayer->currbase;
+   prevexpo = currlayer->currexpo;
    
    if (!inscript) {
       // make sure Undo and Redo items show correct actions while generating
@@ -671,12 +716,13 @@ void UndoRedo::RememberGenStart()
             if (change->changeid == setgen &&
                 change->oldtempstart != change->newtempstart) {
                change->newdirty = currlayer->startdirty;
-               change->newhash = currlayer->starthash;
+               change->newalgo = currlayer->startalgo;
                change->newrule = currlayer->startrule;
                change->newx = currlayer->startx;
                change->newy = currlayer->starty;
                change->newmag = currlayer->startmag;
-               change->newwarp = currlayer->startwarp;
+               change->newbase = currlayer->startbase;
+               change->newexpo = currlayer->startexpo;
                change->newsel = currlayer->startsel;
                change->newname = currlayer->startname;
                if (currlayer->cloneid > 0) {
@@ -697,13 +743,8 @@ void UndoRedo::RememberGenStart()
       }
       
    } else {
-      // save starting pattern in a unique temporary file;
-      // on my Mac the file is in /private/var/tmp/folders.502/TemporaryItems;
-      // on WinXP the file is in C:\Documents and Settings\Andy\Local Settings\Temp
-      // (or shorter equivalent C:\DOCUME~1\Andy\LOCALS~1\Temp) but the file name
-      // is gol*.tmp (ie. only 1st 3 chars of temp_prefix are used);
-      // on Linux the file is in /tmp
-      prevfile = wxFileName::CreateTempFileName(temp_prefix);
+      // save starting pattern in a unique temporary file
+      prevfile = wxFileName::CreateTempFileName(tempdir + temp_prefix);
 
       // if head of undo list is a genchange node then we can copy that
       // change node's newfile to prevfile; this makes consecutive generating
@@ -742,7 +783,7 @@ void UndoRedo::RememberGenFinish()
 
    if (inscript && savegenchanges) return;   // ignore consecutive run/step command
 
-   // generation count might not have changed (can happen in wxGTK)
+   // generation count might not have changed (can happen in Linux app)
    if (prevgen == currlayer->algo->getGeneration()) {
       // delete prevfile created by RememberGenStart
       if (!prevfile.IsEmpty() && wxFileExists(prevfile)) {
@@ -758,7 +799,7 @@ void UndoRedo::RememberGenFinish()
       fpath = wxEmptyString;
    } else {
       // save finishing pattern in a unique temporary file
-      fpath = wxFileName::CreateTempFileName(temp_prefix);
+      fpath = wxFileName::CreateTempFileName(tempdir + temp_prefix);
       SaveCurrentPattern(fpath);
    }
    
@@ -781,8 +822,10 @@ void UndoRedo::RememberGenFinish()
    viewptr->GetPos(change->newx, change->newy);
    change->oldmag = prevmag;
    change->newmag = viewptr->GetMag();
-   change->oldwarp = prevwarp;
-   change->newwarp = currlayer->warp;
+   change->oldbase = prevbase;
+   change->newbase = currlayer->currbase;
+   change->oldexpo = prevexpo;
+   change->newexpo = currlayer->currexpo;
    change->oldsel = prevsel;
    change->newsel = currlayer->currsel;
 
@@ -809,7 +852,8 @@ void UndoRedo::AddGenChange()
    prevx = currlayer->startx;
    prevy = currlayer->starty;
    prevmag = currlayer->startmag;
-   prevwarp = currlayer->startwarp;
+   prevbase = currlayer->startbase;
+   prevexpo = currlayer->startexpo;
    prevfile = wxEmptyString;
 
    // play safe and pretend RememberGenStart was called
@@ -828,7 +872,8 @@ void UndoRedo::AddGenChange()
 void UndoRedo::SyncUndoHistory()
 {
    // synchronize undo history due to a ResetPattern call;
-   // wind back the undo list to just past the oldest genchange node
+   // wind back the undo list to just past the genchange node that
+   // matches the current layer's starting gen count
    wxList::compatibility_iterator node;
    ChangeNode* change;
    while (!undolist.IsEmpty()) {
@@ -871,7 +916,7 @@ void UndoRedo::RememberSetGen(bigint& oldgen, bigint& newgen,
    if (oldgen > oldstartgen && newgen <= oldstartgen) {
       // if pattern is generated then tempstart will be clobbered by
       // SaveStartingPattern, so change tempstart to a new temporary file
-      currlayer->tempstart = wxFileName::CreateTempFileName(_("golly_setgen_"));
+      currlayer->tempstart = wxFileName::CreateTempFileName(tempdir + wxT("golly_setgen_"));
 
       // also need to update startfile and currfile (currlayer->savestart is true)
       currlayer->startfile = currlayer->tempstart;
@@ -904,12 +949,13 @@ void UndoRedo::RememberSetGen(bigint& oldgen, bigint& newgen,
       // save extra starting info set by previous SaveStartingPattern so that
       // Undoing this setgen change will restore the correct info for a Reset
       change->olddirty = currlayer->startdirty;
-      change->oldhash = currlayer->starthash;
+      change->oldalgo = currlayer->startalgo;
       change->oldrule = currlayer->startrule;
       change->oldx = currlayer->startx;
       change->oldy = currlayer->starty;
       change->oldmag = currlayer->startmag;
-      change->oldwarp = currlayer->startwarp;
+      change->oldbase = currlayer->startbase;
+      change->oldexpo = currlayer->startexpo;
       change->oldsel = currlayer->startsel;
       change->oldname = currlayer->startname;
       if (currlayer->cloneid > 0) {
@@ -925,12 +971,13 @@ void UndoRedo::RememberSetGen(bigint& oldgen, bigint& newgen,
       // Redoing this setgen change will restore the correct info for a Reset
       fixsetgen = true;
       change->newdirty = currlayer->startdirty;
-      change->newhash = currlayer->starthash;
+      change->newalgo = currlayer->startalgo;
       change->newrule = currlayer->startrule;
       change->newx = currlayer->startx;
       change->newy = currlayer->starty;
       change->newmag = currlayer->startmag;
-      change->newwarp = currlayer->startwarp;
+      change->newbase = currlayer->startbase;
+      change->newexpo = currlayer->startexpo;
       change->newsel = currlayer->startsel;
       change->newname = currlayer->startname;
       if (currlayer->cloneid > 0) {
@@ -975,9 +1022,8 @@ void UndoRedo::RememberNameChange(const wxString& oldname, const wxString& oldcu
    change->olddirty = olddirty;
    change->newdirty = currlayer->dirty;
    
-   // cloned layers share the same undo/redo history but each clone
-   // can have a different name, so we need to remember which layer
-   // was changed
+   // cloned layers share the same undo/redo history but each clone can have
+   // a different name, so we need to remember which layer was changed
    change->whichlayer = currlayer;
    
    undolist.Insert(change);
@@ -1035,6 +1081,26 @@ void UndoRedo::RememberRuleChange(const wxString& oldrule)
    change->oldrule = oldrule;
    change->newrule = newrule;
    
+   // SaveCellChange may have been called
+   if (numchanges > 0) {
+      if (numchanges < maxchanges) {
+         // reduce size of cellarray
+         cell_change* newptr =
+            (cell_change*) realloc(cellarray, numchanges * sizeof(cell_change));
+         if (newptr != NULL) cellarray = newptr;
+      }
+
+      change->cellinfo = cellarray;
+      change->cellcount = numchanges;
+      
+      numchanges = 0;      // reset for next SaveCellChange
+      maxchanges = 0;      // ditto
+      if (badalloc) {
+         Warning(lack_of_memory);
+         badalloc = false;
+      }
+   }
+   
    undolist.Insert(change);
    
    // update Undo item in Edit menu
@@ -1043,7 +1109,7 @@ void UndoRedo::RememberRuleChange(const wxString& oldrule)
 
 // -----------------------------------------------------------------------------
 
-void UndoRedo::RememberAlgoChange()
+void UndoRedo::RememberAlgoChange(algo_type oldalgo, const wxString& oldrule)
 {
    // clear the redo history
    WX_CLEAR_LIST(wxList, redolist);
@@ -1053,7 +1119,31 @@ void UndoRedo::RememberAlgoChange()
    ChangeNode* change = new ChangeNode(algochange);
    if (change == NULL) Fatal(_("Failed to create algochange node!"));
 
-   change->suffix = _("Hashing Change");
+   change->suffix = _("Algorithm Change");
+   change->oldalgo = oldalgo;
+   change->newalgo = currlayer->algtype;
+   change->oldrule = oldrule;
+   change->newrule = wxString(currlayer->algo->getrule(), wxConvLocal);
+   
+   // SaveCellChange may have been called
+   if (numchanges > 0) {
+      if (numchanges < maxchanges) {
+         // reduce size of cellarray
+         cell_change* newptr =
+            (cell_change*) realloc(cellarray, numchanges * sizeof(cell_change));
+         if (newptr != NULL) cellarray = newptr;
+      }
+
+      change->cellinfo = cellarray;
+      change->cellcount = numchanges;
+      
+      numchanges = 0;      // reset for next SaveCellChange
+      maxchanges = 0;      // ditto
+      if (badalloc) {
+         Warning(lack_of_memory);
+         badalloc = false;
+      }
+   }
    
    undolist.Insert(change);
    
@@ -1390,13 +1480,13 @@ bool CopyTempFiles(ChangeNode* srcnode, ChangeNode* destnode, const wxString& te
    bool allcopied = true;
 
    if ( !srcnode->oldfile.IsEmpty() && wxFileExists(srcnode->oldfile) ) {
-      destnode->oldfile = wxFileName::CreateTempFileName(_("golly_dupe1_"));
+      destnode->oldfile = wxFileName::CreateTempFileName(tempdir + wxT("golly_dupe1_"));
       if ( !wxCopyFile(srcnode->oldfile, destnode->oldfile, true) )
          allcopied = false;
    }
    
    if ( !srcnode->newfile.IsEmpty() && wxFileExists(srcnode->newfile) ) {
-      destnode->newfile = wxFileName::CreateTempFileName(_("golly_dupe2_"));
+      destnode->newfile = wxFileName::CreateTempFileName(tempdir + wxT("golly_dupe2_"));
       if ( !wxCopyFile(srcnode->newfile, destnode->newfile, true) )
          allcopied = false;
    }
@@ -1406,7 +1496,7 @@ bool CopyTempFiles(ChangeNode* srcnode, ChangeNode* destnode, const wxString& te
          // the file has already been copied to tempstart1 by Layer::Layer()
          destnode->oldtempstart = tempstart1;
       } else {
-         destnode->oldtempstart = wxFileName::CreateTempFileName(_("golly_dupe3_"));
+         destnode->oldtempstart = wxFileName::CreateTempFileName(tempdir + wxT("golly_dupe3_"));
          if ( !wxCopyFile(srcnode->oldtempstart, destnode->oldtempstart, true) )
             allcopied = false;
       }
@@ -1421,7 +1511,7 @@ bool CopyTempFiles(ChangeNode* srcnode, ChangeNode* destnode, const wxString& te
          // the file has already been copied to tempstart1 by Layer::Layer()
          destnode->newtempstart = tempstart1;
       } else {
-         destnode->newtempstart = wxFileName::CreateTempFileName(_("golly_dupe4_"));
+         destnode->newtempstart = wxFileName::CreateTempFileName(tempdir + wxT("golly_dupe4_"));
          if ( !wxCopyFile(srcnode->newtempstart, destnode->newtempstart, true) )
             allcopied = false;
       }
@@ -1436,31 +1526,37 @@ bool CopyTempFiles(ChangeNode* srcnode, ChangeNode* destnode, const wxString& te
 
 // -----------------------------------------------------------------------------
 
-void UndoRedo::Duplicate(UndoRedo* history, const wxString& tempstart)
+void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
 {
-   // clear any current history
-   ClearUndoRedo();
+   UndoRedo* history = oldlayer->undoredo;
+   
+   // clear the undo/redo lists; note that UndoRedo::UndoRedo has added
+   // a scriptstart node to undolist if inscript is true, but we don't
+   // want that here because the old layer's history will already have one
+   WX_CLEAR_LIST(wxList, undolist);
+   WX_CLEAR_LIST(wxList, redolist);
    
    // safer to do our own shallow copy (avoids setting undolist/redolist)
    savecellchanges = history->savecellchanges;
    savegenchanges = history->savegenchanges;
    doingscriptchanges = history->doingscriptchanges;
-   intcount = history->intcount;
-   maxcount = history->maxcount;
+   numchanges = history->numchanges;
+   maxchanges = history->maxchanges;
    badalloc = history->badalloc;
    prevfile = history->prevfile;
    prevgen = history->prevgen;
    prevx = history->prevx;
    prevy = history->prevy;
    prevmag = history->prevmag;
-   prevwarp = history->prevwarp;
+   prevbase = history->prevbase;
+   prevexpo = history->prevexpo;
    prevsel = history->prevsel;
    startcount = history->startcount;
    fixsetgen = history->fixsetgen;
    
    // copy existing temporary file to new name
    if ( !prevfile.IsEmpty() && wxFileExists(prevfile) ) {
-      prevfile = wxFileName::CreateTempFileName(temp_prefix);
+      prevfile = wxFileName::CreateTempFileName(tempdir + temp_prefix);
       if ( !wxCopyFile(history->prevfile, prevfile, true) ) {
          Warning(_("Could not copy prevfile!"));
          return;
@@ -1469,14 +1565,14 @@ void UndoRedo::Duplicate(UndoRedo* history, const wxString& tempstart)
 
    // do a deep copy of dynamically allocated data
    cellarray = NULL;
-   if (intcount > 0 && history->cellarray) {
-      cellarray = (int*) malloc(maxcount * sizeof(int));
+   if (numchanges > 0 && history->cellarray) {
+      cellarray = (cell_change*) malloc(maxchanges * sizeof(cell_change));
       if (cellarray == NULL) {
          Warning(_("Could not allocate cellarray!"));
          return;
       }
       // copy history->cellarray data to this cellarray
-      memcpy(cellarray, history->cellarray, intcount * sizeof(int));
+      memcpy(cellarray, history->cellarray, numchanges * sizeof(cell_change));
    } 
 
    wxList::compatibility_iterator node;
@@ -1497,24 +1593,29 @@ void UndoRedo::Duplicate(UndoRedo* history, const wxString& tempstart)
       *newchange = *change;
       
       // deep copy any dynamically allocated data
-      if (change->cellcoords) {
-         int bytes = change->cellcount * 2 * sizeof(int);
-         newchange->cellcoords = (int*) malloc(bytes);
-         if (newchange->cellcoords == NULL) {
+      if (change->cellinfo) {
+         int bytes = change->cellcount * sizeof(cell_change);
+         newchange->cellinfo = (cell_change*) malloc(bytes);
+         if (newchange->cellinfo == NULL) {
             Warning(_("Could not copy undolist!"));
             WX_CLEAR_LIST(wxList, undolist);
             return;
          }
-         memcpy(newchange->cellcoords, change->cellcoords, bytes);
+         memcpy(newchange->cellinfo, change->cellinfo, bytes);
       }
       
       // copy any existing temporary files to new names
-      if (!CopyTempFiles(change, newchange, tempstart)) {
+      if (!CopyTempFiles(change, newchange, newlayer->tempstart)) {
          Warning(_("Failed to copy temporary file in undolist!"));
          WX_CLEAR_LIST(wxList, undolist);
          return;
       }
       
+      // if node is a name change then update whichlayer to point to new layer
+      if (newchange->changeid == namechange) {
+         newchange->whichlayer = newlayer;
+      }
+      
       undolist.Append(newchange);
       node = node->GetNext();
    }
@@ -1535,24 +1636,29 @@ void UndoRedo::Duplicate(UndoRedo* history, const wxString& tempstart)
       *newchange = *change;
       
       // deep copy any dynamically allocated data
-      if (change->cellcoords) {
-         int bytes = change->cellcount * 2 * sizeof(int);
-         newchange->cellcoords = (int*) malloc(bytes);
-         if (newchange->cellcoords == NULL) {
+      if (change->cellinfo) {
+         int bytes = change->cellcount * sizeof(cell_change);
+         newchange->cellinfo = (cell_change*) malloc(bytes);
+         if (newchange->cellinfo == NULL) {
             Warning(_("Could not copy redolist!"));
             WX_CLEAR_LIST(wxList, redolist);
             return;
          }
-         memcpy(newchange->cellcoords, change->cellcoords, bytes);
+         memcpy(newchange->cellinfo, change->cellinfo, bytes);
       }
       
       // copy any existing temporary files to new names
-      if (!CopyTempFiles(change, newchange, tempstart)) {
+      if (!CopyTempFiles(change, newchange, newlayer->tempstart)) {
          Warning(_("Failed to copy temporary file in redolist!"));
          WX_CLEAR_LIST(wxList, redolist);
          return;
       }
       
+      // if node is a name change then update whichlayer to point to new layer
+      if (newchange->changeid == namechange) {
+         newchange->whichlayer = newlayer;
+      }
+      
       redolist.Append(newchange);
       node = node->GetNext();
    }
