@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2008 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -58,6 +58,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 void Note(const wxString& msg)
 {
    wxString title = wxGetApp().GetAppName() + _(" note:");
+   #ifdef __WXMAC__
+      wxSetCursor(*wxSTANDARD_CURSOR);
+   #endif
    wxMessageBox(msg, title, wxOK | wxICON_INFORMATION, wxGetActiveWindow());
 }
 
@@ -67,6 +70,9 @@ void Warning(const wxString& msg)
 {
    wxBell();
    wxString title = wxGetApp().GetAppName() + _(" warning:");
+   #ifdef __WXMAC__
+      wxSetCursor(*wxSTANDARD_CURSOR);
+   #endif
    wxMessageBox(msg, title, wxOK | wxICON_EXCLAMATION, wxGetActiveWindow());
 }
 
@@ -76,6 +82,9 @@ void Fatal(const wxString& msg)
 {
    wxBell();
    wxString title = wxGetApp().GetAppName() + _(" error:");
+   #ifdef __WXMAC__
+      wxSetCursor(*wxSTANDARD_CURSOR);
+   #endif
    wxMessageBox(msg, title, wxOK | wxICON_ERROR, wxGetActiveWindow());
    // calling wxExit() results in a bus error on X11
    exit(1);
@@ -216,7 +225,12 @@ public:
 void IntegerDialog::OnSpinCtrlChar(wxKeyEvent& event)
 {
    int key = event.GetKeyCode();
-   if ( key == WXK_TAB ) {
+   
+   if (event.CmdDown()) {
+      // allow handling of cmd-x/v/etc
+      event.Skip();
+
+   } else if ( key == WXK_TAB ) {
       /* why does this crash???!!!
       spinctrl->SetFocus();
       spinctrl->SetSelection(0,999);
@@ -226,6 +240,7 @@ void IntegerDialog::OnSpinCtrlChar(wxKeyEvent& event)
          sc->SetFocus();
          sc->SetSelection(0,999);
       }
+
    } else if ( key >= ' ' && key <= '~' ) {
       if ( (key >= '0' && key <= '9') || key == '+' || key == '-' ) {
          // allow digits and + or -
@@ -234,6 +249,7 @@ void IntegerDialog::OnSpinCtrlChar(wxKeyEvent& event)
          // disallow any other displayable ascii char
          wxBell();
       }
+
    } else {
       event.Skip();
    }
@@ -484,7 +500,7 @@ void BeginProgress(const wxString& dlgtitle)
    #ifdef __WXMAC__
       wxSetCursor(*wxHOURGLASS_CURSOR);
    #endif
-   viewptr->SetCursor(*wxHOURGLASS_CURSOR);
+   if (mainptr->IsActive()) viewptr->SetCursor(*wxHOURGLASS_CURSOR);
 }
 
 // -----------------------------------------------------------------------------
@@ -495,11 +511,18 @@ bool AbortProgress(double fraction_done, const wxString& newmsg)
    if (progdlg) {
       if (msecs < prognext) return false;
       #ifdef __WXX11__
-         prognext = msecs + 1000;    // call Update about once per sec on X11
+         prognext = msecs + 1000;    // call Update/Pulse about once per sec on X11
       #else
-         prognext = msecs + 100;     // call Update about 10 times per sec
+         prognext = msecs + 100;     // call Update/Pulse about 10 times per sec
       #endif
-      if ( !progdlg->Update(int((double)maxprogrange * fraction_done), newmsg) ) {
+      bool cancel;
+      if (fraction_done < 0.0) {
+         // show indeterminate progress gauge
+         cancel = !progdlg->Pulse(newmsg);
+      } else {
+         cancel = !progdlg->Update(int((double)maxprogrange * fraction_done), newmsg);
+      }
+      if (cancel) {
          // user hit Cancel button
          if (inscript) PassKeyToScript(WXK_ESCAPE);     // abort script
          return true;
@@ -512,7 +535,7 @@ bool AbortProgress(double fraction_done, const wxString& newmsg)
       if ( (msecs > 1000 && fraction_done < 0.3) || msecs > 2500 ) {
          // task is probably going to take a while so create progress dialog
          progdlg = new wxProgressDialog(progtitle, wxEmptyString,
-                                        maxprogrange, mainptr,
+                                        maxprogrange, wxGetActiveWindow(),
                                         wxPD_AUTO_HIDE | wxPD_APP_MODAL |
                                         wxPD_CAN_ABORT | wxPD_SMOOTH |
                                         wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME);
@@ -525,7 +548,7 @@ bool AbortProgress(double fraction_done, const wxString& newmsg)
             }
          #endif
       }
-      prognext = msecs + 10;     // short delay until 1st Update
+      prognext = msecs + 10;     // short delay until 1st Update/Pulse
       return false;              // don't abort
    }
 }
@@ -613,3 +636,66 @@ void CreatePaleBitmap(const wxBitmap& inmap, wxBitmap& outmap)
 
    outmap = wxBitmap(newimg);
 }
+
+// -----------------------------------------------------------------------------
+
+bool IsScriptFile(const wxString& filename)
+{
+   wxString ext = filename.AfterLast('.');
+   // if filename has no extension then ext == filename
+   if (ext == filename) return false;
+   // currently we support Perl or Python scripts
+   return ( ext.IsSameAs(wxT("pl"),false) ||
+            ext.IsSameAs(wxT("py"),false) );
+}
+
+// -----------------------------------------------------------------------------
+
+bool IsHTMLFile(const wxString& filename)
+{
+   wxString ext = filename.AfterLast('.');
+   // if filename has no extension then ext == filename
+   if (ext == filename) return false;
+   return ( ext.IsSameAs(wxT("htm"),false) ||
+            ext.IsSameAs(wxT("html"),false) );
+}
+
+// -----------------------------------------------------------------------------
+
+bool IsTextFile(const wxString& filename)
+{
+   if (!IsHTMLFile(filename)) {
+      // if non-html file name contains "readme" then assume it's a text file
+      wxString name = filename.AfterLast(wxFILE_SEP_PATH).MakeLower();
+      if (name.Contains(wxT("readme"))) return true;
+   }
+   wxString ext = filename.AfterLast('.');
+   // if filename has no extension then ext == filename
+   if (ext == filename) return false;
+   return ( ext.IsSameAs(wxT("txt"),false) ||
+            ext.IsSameAs(wxT("doc"),false) );
+}
+
+// -----------------------------------------------------------------------------
+
+bool IsZipFile(const wxString& filename)
+{
+   wxString ext = filename.AfterLast('.');
+   // if filename has no extension then ext == filename
+   if (ext == filename) return false;
+   return ( ext.IsSameAs(wxT("zip"),false) ||
+            ext.IsSameAs(wxT("gar"),false) );
+}
+
+// -----------------------------------------------------------------------------
+
+bool IsRuleFile(const wxString& filename)
+{
+   wxString ext = filename.AfterLast('.');
+   // if filename has no extension then ext == filename
+   if (ext == filename) return false;
+   return ( ext.IsSameAs(wxT("table"),false) ||
+            ext.IsSameAs(wxT("tree"),false) ||
+            ext.IsSameAs(wxT("colors"),false) ||
+            ext.IsSameAs(wxT("icons"),false) );
+}
